@@ -37,7 +37,7 @@ Print the titles of the unread entries::
 
 """
 
-__version__ = '3.5'
+__version__ = '3.12'
 
 from .core import (
     Reader as Reader,
@@ -65,11 +65,15 @@ from .exceptions import (
     FeedError as FeedError,
     FeedExistsError as FeedExistsError,
     FeedNotFoundError as FeedNotFoundError,
-    ParseError as ParseError,
     InvalidFeedURLError as InvalidFeedURLError,
     EntryError as EntryError,
     EntryExistsError as EntryExistsError,
     EntryNotFoundError as EntryNotFoundError,
+    UpdateError as UpdateError,
+    ParseError as ParseError,
+    UpdateHookError as UpdateHookError,
+    SingleUpdateHookError as SingleUpdateHookError,
+    UpdateHookErrorGroup as UpdateHookErrorGroup,
     StorageError as StorageError,
     SearchError as SearchError,
     SearchNotEnabledError as SearchNotEnabledError,
