@@ -6,6 +6,7 @@ import time
 import typing
 from dataclasses import dataclass
 from datetime import datetime
+from datetime import timedelta
 from datetime import timezone
 
 import flask.signals
@@ -47,8 +48,14 @@ blueprint = Blueprint('reader', __name__)
 def humanize_naturaltime(dt):
     when = None
     if dt.tzinfo:
-        when = datetime.utcnow().replace(tzinfo=timezone.utc)
-    return humanize.naturaltime(dt, when=when)
+        when = datetime.now(tz=timezone.utc)
+    try:
+        return humanize.naturaltime(dt, when=when)
+    except ValueError as e:
+        # can happen for 0001-01-01
+        if 'year 0 is out of range' not in str(e):
+            raise
+        return humanize.naturaltime(dt + timedelta(days=1), when=when)
 
 
 @blueprint.app_template_filter()
@@ -110,6 +117,24 @@ def add_reader_version():
     g.reader_version = reader.__version__
 
 
+@blueprint.before_request
+def enable_reader_timer():
+    g.reader_timer = None
+    # if 'timings' not in request.args: return
+    reader = get_reader()
+    if not hasattr(reader, 'timer'):
+        return
+    g.reader_timer = reader.timer
+    reader.timer.enable()
+
+
+@blueprint.teardown_request
+def close_reader_timer(_):
+    if not g.reader_timer:
+        return
+    g.reader_timer.disable()
+
+
 def highlighted(string):
     # needs to be marked as safe so we don't need to do it everywhere in the template
     # TODO: maybe use something "more semantic" than <b> (CSS needs changing too if so)
@@ -265,7 +290,6 @@ def entries():
         # if tags is '', it's not a tag filter
         return redirect(url_for('.entries', **args))
     else:
-
         try:
             tags = yaml.safe_load(tags)
         except yaml.YAMLError as e:
@@ -425,7 +449,6 @@ def feeds():
         # if tags is '', it's not a tag filter
         return redirect(url_for('.feeds', **args))
     else:
-
         try:
             tags = yaml.safe_load(tags)
         except yaml.YAMLError as e:
@@ -783,6 +806,13 @@ def additional_enclosure_links(enclosure, entry):
         yield from func(enclosure, entry)
 
 
+@blueprint.app_template_global()
+def additional_links(entry):
+    funcs = getattr(current_app, 'reader_additional_links', ())
+    for func in funcs:
+        yield from func(entry)
+
+
 def create_app(config):
     app = Flask(__name__)
     app.secret_key = 'secret'
@@ -793,6 +823,7 @@ def create_app(config):
 
     # NOTE: this is part of the app extension API
     app.reader_additional_enclosure_links = []
+    app.reader_additional_links = []
 
     app.plugin_loader = loader = Loader()
 
