@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import logging
+from collections.abc import Callable
 from collections.abc import Iterable
 from dataclasses import dataclass
 from datetime import datetime
@@ -9,7 +10,6 @@ from itertools import chain
 from itertools import starmap
 from itertools import tee
 from typing import Any
-from typing import Callable
 from typing import Optional
 from typing import TYPE_CHECKING
 
@@ -20,10 +20,12 @@ from ._types import FeedData
 from ._types import FeedForUpdate
 from ._types import FeedUpdateIntent
 from ._types import ParsedFeed
+from ._types import UpdateHooks
 from ._utils import count_consumed
 from ._utils import PrefixLogger
 from .exceptions import FeedNotFoundError
 from .exceptions import ParseError
+from .exceptions import UpdateError
 from .types import EntryUpdateStatus
 from .types import ExceptionInfo
 from .types import UpdatedFeed
@@ -31,11 +33,9 @@ from .types import UpdateResult
 
 if TYPE_CHECKING:  # pragma: no cover
     from ._parser import Parser
-    from ._storage import Storage
-    from ._types import EntryUpdateIntent
-    from ._types import FeedFilterOptions
-    from ._types import FeedUpdateIntent
-    from ._utils import MapType
+    from ._types import FeedFilter
+    from ._types import StorageType
+    from ._utils import MapFunction
     from .core import Reader
 
 
@@ -205,7 +205,6 @@ class Decider:
 
     def get_entries_to_update(self, pairs: EntryPairs) -> Iterable[EntryUpdateIntent]:
         for feed_order, (new, old) in reversed(list(enumerate(pairs))):
-
             # This may fail if we ever implement changing the feed URL
             # in response to a permanent redirect.
             assert new.feed_url == self.url, f'{new.feed_url!r}, {self.url!r}'
@@ -264,10 +263,8 @@ class Decider:
         parsed_feed: ParsedFeed | None | ParseError,
         entry_pairs: EntryPairs,
     ) -> tuple[FeedUpdateIntent | None, Iterable[EntryUpdateIntent]]:
-
         # Not modified.
         if not parsed_feed:
-
             # New feed shouldn't be considered new anymore.
             if not self.old_feed.last_updated:
                 return FeedUpdateIntent(self.url, self.now), ()
@@ -279,9 +276,7 @@ class Decider:
             return None, ()
 
         if isinstance(parsed_feed, ParseError):
-            exc_info = ExceptionInfo.from_exception(
-                parsed_feed.__cause__ or parsed_feed
-            )
+            exc_info = ExceptionInfo.from_exception(parsed_feed)
             return FeedUpdateIntent(self.url, None, last_exception=exc_info), ()
 
         entries_to_update = list(self.get_entries_to_update(entry_pairs))
@@ -320,26 +315,24 @@ class Pipeline:
 
     """
 
-    storage: Storage
+    storage: StorageType
     parser: Parser
+    hooks: UpdateHooks[Any]
     now: Callable[[], datetime]
-    map: MapType
-    # for hooks' usage *only*
-    reader: Reader
+    map: MapFunction[Any, Any]
     decider = Decider
 
     @classmethod
-    def from_reader(cls, reader: Reader, map: MapType) -> Pipeline:
+    def from_reader(cls, reader: Reader, map: MapFunction[Any, Any]) -> Pipeline:
         return cls(
             storage=reader._storage,
             parser=reader._parser,
+            hooks=reader._update_hooks,
             now=reader._now,
             map=map,
-            reader=reader,
         )
 
-    def update(self, filter_options: FeedFilterOptions) -> Iterable[UpdateResult]:
-
+    def update(self, filter: FeedFilter) -> Iterable[UpdateResult]:
         # global_now is used as first_updated_epoch for all new entries,
         # so that the subset of new entries from an update appears before
         # all others and the entries in it are sorted by published/updated;
@@ -377,7 +370,7 @@ class Pipeline:
                     parser_process_feeds_for_update_errors.append((feed, e))
 
         # assemble pipeline
-        feeds_for_update = self.storage.get_feeds_for_update(filter_options)
+        feeds_for_update = self.storage.get_feeds_for_update(filter)
         # feeds_for_update = map(self.parser.process_feed_for_update, feeds_for_update)
         feeds_for_update = parser_process_feeds_for_update(feeds_for_update)
         feeds_for_update = map(self.decider.process_feed_for_update, feeds_for_update)
@@ -391,7 +384,7 @@ class Pipeline:
                 continue
 
             if isinstance(value, Exception):
-                if not isinstance(value, ParseError):
+                if not isinstance(value, UpdateError):
                     raise value
 
             yield UpdateResult(url, value)
@@ -402,7 +395,6 @@ class Pipeline:
         feed: FeedForUpdate,
         result: ParsedFeed | None | ParseError,
     ) -> tuple[str, UpdatedFeed | None | Exception]:
-
         make_intents = partial(
             self.decider.make_intents, feed, self.now(), global_now, result
         )
@@ -440,7 +432,7 @@ class Pipeline:
         entries_for_update = self.storage.get_entries_for_update(
             (e.feed_url, e.id) for e in entries1
         )
-        return zip(entries2, entries_for_update)
+        return zip(entries2, entries_for_update, strict=True)
 
     def update_feed(
         self,
@@ -448,9 +440,7 @@ class Pipeline:
         feed: FeedUpdateIntent | None,
         entries: Iterable[EntryUpdateIntent],
     ) -> tuple[int, int]:
-
-        for feed_hook in self.reader.before_feed_update_hooks:
-            feed_hook(self.reader, url)
+        self.hooks.run('before_feed_update', (url,), url)
 
         if feed:
             if entries:
@@ -460,6 +450,8 @@ class Pipeline:
         # if feed_for_update.url != parsed_feed.feed.url, the feed was redirected.
         # TODO: Maybe handle redirects somehow else (e.g. change URL if permanent).
 
+        hook_errors = self.hooks.group("got unexpected after-update hook errors")
+
         new_count = 0
         updated_count = 0
         for entry in entries:
@@ -469,10 +461,16 @@ class Pipeline:
             else:
                 updated_count += 1
                 entry_status = EntryUpdateStatus.MODIFIED
-            for entry_hook in self.reader.after_entry_update_hooks:
-                entry_hook(self.reader, entry.entry, entry_status)
 
-        for feed_hook in self.reader.after_feed_update_hooks:
-            feed_hook(self.reader, url)
+            hook_errors.run(
+                'after_entry_update',
+                entry.entry.resource_id,
+                entry.entry,
+                entry_status,
+                limit=5,
+            )
+
+        hook_errors.run('after_feed_update', (url,), url)
+        hook_errors.close()
 
         return new_count, updated_count
