@@ -4,17 +4,17 @@ import functools
 import inspect
 import itertools
 import logging
+import pkgutil
 import warnings
+from collections.abc import Callable
 from collections.abc import Iterable
 from collections.abc import Iterator
 from collections.abc import Sequence
+from contextlib import AbstractContextManager
 from contextlib import contextmanager
 from functools import wraps
-from queue import Queue
 from typing import Any
-from typing import Callable
 from typing import cast
-from typing import no_type_check
 from typing import TypeVar
 
 
@@ -58,66 +58,6 @@ def exactly_one(it: Iterable[_U]) -> _U:
         assert False, "shouldn't get here"  # noqa: B011; # pragma: no cover
 
 
-def join_paginated_iter(
-    get_things: Callable[[int, _T | None], Iterable[tuple[_U, _T]]],
-    chunk_size: int,
-    last: _T | None = None,
-    limit: int = 0,
-) -> Iterable[_U]:
-    """
-    count_to_ten(4, None) -> ('one', 1), ..., ('four', 4)
-    count_to_ten(0, None) -> ('one', 1), ..., ('ten', 10)
-    count_to_ten(4, 4) -> ('five', 5), ..., ('eight', 8)
-    count_to_ten(0, 4) -> ('five', 5), ..., ('ten', 10)
-
-    join_paginated_iter(count_to_ten, 4, None) -> one, ..., ten (3 calls)
-    join_paginated_iter(count_to_ten, 0, None) -> one, ..., ten (1 call)
-    join_paginated_iter(count_to_ten, 4, 4) -> five, ..., ten (2 calls)
-    join_paginated_iter(count_to_ten, 0, 4) -> five, ..., ten (1 call)
-    join_paginated_iter(count_to_ten, 4, 4, limit=5) -> five, ..., nine (2 calls)
-    join_paginated_iter(count_to_ten, 0, 4, limit=5) -> five, ..., nine (1 call)
-
-    """
-    # At the moment get_things must take positional arguments.
-    # We could make it work with kwargs by using protocols,
-    # but mypy gets confused about partials with kwargs.
-    # https://github.com/python/mypy/issues/1484
-
-    if not chunk_size:
-        # When chunk_size is 0, don't chunk the query.
-        #
-        # This will ensure there are no missing/duplicated entries, but
-        # will block database writes until the whole generator is consumed.
-        #
-        # Currently not exposed through the public API.
-        #
-        things = get_things(limit, last)
-        yield from (t for t, _ in things)
-        return
-
-    remaining = limit
-
-    while True:
-        if limit:
-            if not remaining:
-                break
-            to_get = min(remaining, chunk_size)
-            remaining = max(0, remaining - to_get)
-        else:
-            to_get = chunk_size
-
-        things = list(get_things(to_get, last))
-        if not things:
-            break
-
-        _, last = things[-1]
-
-        yield from (t for t, _ in things)
-
-        if len(things) < to_get:
-            break
-
-
 def chunks(n: int, iterable: Iterable[_T]) -> Iterable[Iterable[_T]]:
     """grouper(2, 'ABCDE') --> AB CD E"""
     # based on https://stackoverflow.com/a/8991553
@@ -146,64 +86,49 @@ def count_consumed(it: Iterable[_T]) -> tuple[Iterable[_T], Callable[[], int]]:
     return wrapper(), get_count
 
 
+MapFunction = Callable[[Callable[[_T], _U], Iterable[_T]], Iterator[_U]]
+MapContextManager = AbstractContextManager[MapFunction[_T, _U]]
+
+
 @contextmanager
-def make_pool_map(workers: int) -> Iterator[F]:
+def make_pool_map(workers: int) -> Iterator[MapFunction[_T, _U]]:
+    # We are using concurrent.futures instead of multiprocessing.dummy
+    # because the latter doesn't work on some environments (e.g. AWS Lambda).
+    # We are not using executor.map() because it consumes the entire iterable.
+
     # lazy import (https://github.com/lemon24/reader/issues/297)
-    import multiprocessing.dummy
+    import concurrent.futures
 
-    pool = multiprocessing.dummy.Pool(workers)
-    try:
-        yield wrap_map(pool.imap_unordered, workers)
-    finally:
-        pool.close()
-        pool.join()
+    executor = concurrent.futures.ThreadPoolExecutor(workers)
 
+    def imap_unordered(fn: Callable[[_T], _U], iterable: Iterable[_T]) -> Iterator[_U]:
+        iterable = iter(iterable)
+        iterable_ended = False
+        pending: set[concurrent.futures.Future[_U]] = set()
 
-def wrap_map(map: F, workers: int) -> F:
-    """Ensure map() calls next() on its iterable in the current thread.
+        while pending or not iterable_ended:
+            while len(pending) < workers and not iterable_ended:
+                try:
+                    arg = next(iterable)
+                except StopIteration:
+                    iterable_ended = True
+                else:
+                    pending.add(executor.submit(fn, arg))
 
-    multiprocessing.dummy.Pool.imap_unordered seems to pass
-    the iterable to the worker threads, which call next() on it.
+            if not pending:  # pragma: no cover
+                return
 
-    For generators, this means the generator code runs in the worker thread,
-    which is a problem if the generator calls stuff that shouldn't be called
-    across threads; e.g., calling a sqlite3.Connection method results in:
+            done, pending = concurrent.futures.wait(
+                pending, return_when=concurrent.futures.FIRST_COMPLETED
+            )
+            while done:
+                yield done.pop().result()
 
-        sqlite3.ProgrammingError: SQLite objects created in a thread
-        can only be used in that same thread. The object was created
-        in thread id 1234 and this is thread id 5678.
-
-    """
-
-    @no_type_check
-    def wrapper(func, iterable):
-        sentinel = object()
-        queue = Queue()
-
-        for _ in range(workers):
-            queue.put(next(iterable, sentinel))
-
-        for rv in map(func, iter(queue.get, sentinel)):
-            queue.put(next(iterable, sentinel))
-            yield rv
-
-    return cast(F, wrapper)
-
-
-# The type of map() should be
-#
-#   Callable[[Callable[[_T], _U], Iterable[_T]], Iterator[_U]]
-#
-# but mypy gets confused; known issue:
-#
-# https://github.com/python/mypy/issues/1317
-# https://github.com/python/mypy/issues/6697
-#
-MapType = Callable[[Callable[[Any], Any], Iterable[Any]], Iterator[Any]]
+    with executor:
+        yield imap_unordered
 
 
 class PrefixLogger(logging.LoggerAdapter):  # type: ignore
-
     # if needed, add: with log.push('another prefix'): ...
 
     def __init__(self, logger: logging.Logger, prefixes: Sequence[str] = ()):
@@ -259,7 +184,11 @@ def _deprecated_wrapper(
 
     @wraps(func)
     def old_func(*args, **kwargs):  # type: ignore
-        warnings.warn(warning_template.format_map(format_kwargs), DeprecationWarning)
+        warnings.warn(
+            warning_template.format_map(format_kwargs),
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return func(*args, **kwargs)
 
     old_func.__name__ = old_name
@@ -316,3 +245,12 @@ class BetterStrPartial(functools.partial[_T]):
             f"{k}={getattr(v, 'resource_id', v)!r}" for k, v in self.keywords.items()
         )
         return f"{name}({', '.join(parts)})"
+
+
+def lazy_import(module: str, names: list[str]) -> Callable[[str], Any]:
+    def __getattr__(name: str) -> Any:
+        if name not in names:  # pragma: no cover
+            raise AttributeError(f"module {module!r} has no attribute {name!r}")
+        return pkgutil.resolve_name(f'{module}._lazy:{name}')
+
+    return __getattr__
