@@ -4,6 +4,7 @@ import builtins
 import logging
 import numbers
 import warnings
+from collections.abc import Callable
 from collections.abc import Iterable
 from collections.abc import Mapping
 from collections.abc import MutableSequence
@@ -12,27 +13,29 @@ from datetime import datetime
 from datetime import timezone
 from types import MappingProxyType
 from typing import Any
-from typing import Callable
 from typing import Literal
 from typing import overload
+from typing import TYPE_CHECKING
 from typing import TypeVar
 
 from ._parser import default_parser
-from ._parser import Parser
-from ._requests_utils import DEFAULT_TIMEOUT
-from ._requests_utils import TimeoutType
-from ._search import Search
+from ._parser.requests import DEFAULT_TIMEOUT
+from ._parser.requests import TimeoutType
 from ._storage import Storage
+from ._types import BoundSearchStorageType
 from ._types import DEFAULT_RESERVED_NAME_SCHEME
 from ._types import entry_data_from_obj
 from ._types import EntryData
-from ._types import EntryFilterOptions
+from ._types import EntryFilter
 from ._types import EntryUpdateIntent
-from ._types import FeedFilterOptions
-from ._types import fix_datetime_tzinfo
+from ._types import FeedFilter
 from ._types import NameScheme
+from ._types import SearchType
+from ._types import StorageType
+from ._types import UpdateHooks
 from ._update import Pipeline
 from ._utils import make_pool_map
+from ._utils import MapContextManager
 from ._utils import zero_or_one
 from .exceptions import EntryNotFoundError
 from .exceptions import FeedExistsError
@@ -40,7 +43,10 @@ from .exceptions import FeedNotFoundError
 from .exceptions import ParseError
 from .exceptions import PluginInitError
 from .exceptions import SearchNotEnabledError
+from .exceptions import SingleUpdateHookError
 from .exceptions import TagNotFoundError
+from .exceptions import UpdateHookError
+from .exceptions import UpdateHookErrorGroup
 from .plugins import _load_plugins
 from .plugins import DEFAULT_PLUGINS
 from .plugins import PluginInput
@@ -54,12 +60,12 @@ from .types import EntryCounts
 from .types import EntryInput
 from .types import EntrySearchCounts
 from .types import EntrySearchResult
-from .types import EntrySortOrder
+from .types import EntrySort
 from .types import EntryUpdateStatus
 from .types import Feed
 from .types import FeedCounts
 from .types import FeedInput
-from .types import FeedSortOrder
+from .types import FeedSort
 from .types import JSONType
 from .types import MISSING
 from .types import MissingType
@@ -70,14 +76,16 @@ from .types import TristateFilterInput
 from .types import UpdatedFeed
 from .types import UpdateResult
 
+if TYPE_CHECKING:  # pragma: no cover
+    from typing_extensions import Self
+    from ._parser import Parser
+
 
 log = logging.getLogger('reader')
 
 
 _T = TypeVar('_T')
 _U = TypeVar('_U')
-# mypy doesn't seem to support Self yet.
-_TReader = TypeVar('_TReader', bound='Reader')
 
 AfterEntryUpdateHook = Callable[['Reader', EntryData, EntryUpdateStatus], None]
 FeedUpdateHook = Callable[['Reader', str], None]
@@ -92,8 +100,7 @@ def make_reader(
     session_timeout: TimeoutType = DEFAULT_TIMEOUT,
     reserved_name_scheme: Mapping[str, str] = DEFAULT_RESERVED_NAME_SCHEME,
     search_enabled: bool | None | Literal['auto'] = 'auto',
-    _storage: Storage | None = None,
-    _storage_factory: Any = None,
+    _storage: StorageType | None = None,
 ) -> Reader:
     """Create a new :class:`Reader`.
 
@@ -250,14 +257,15 @@ def make_reader(
     # See this comment for details on how it should evolve:
     # https://github.com/lemon24/reader/issues/168#issuecomment-642002049
 
-    storage = _storage or Storage(url, factory=_storage_factory)
+    storage: StorageType = _storage or Storage(url)
 
     try:
         # For now, we're using a storage-bound search provider.
-        search = Search(storage)
+        if not isinstance(storage, BoundSearchStorageType):  # pragma: no cover
+            raise TypeError("storage must have a make_search factory")
+        search = storage.make_search()
 
         if search_enabled is True:
-            search.check_dependencies()
             search.enable()
         elif search_enabled is False:
             search.disable()
@@ -349,107 +357,32 @@ class Reader:
 
     def __init__(
         self,
-        _storage: Storage,
-        _search: Search,
+        _storage: StorageType,
+        _search: SearchType,
         _parser: Parser,
         _reserved_name_scheme: NameScheme,
         _enable_search: bool = False,
         _called_directly: bool = True,
     ):
+        #: The :class:`~reader._types.StorageType` instance used by this reader.
         self._storage = _storage
+        #: The :class:`~reader._types.SearchType` instance used by this reader.
         self._search = _search
+        #: The :class:`~reader._parser.Parser` instance used by this reader.
         self._parser = _parser
 
         self._reserved_name_scheme = _reserved_name_scheme
-
         self._enable_search = _enable_search
-
-        #: List of functions called for each updated entry
-        #: after the feed is updated.
-        #:
-        #: Each function is called with:
-        #:
-        #: * `reader` – the :class:`Reader` instance
-        #: * `entry` – an :class:`Entry`-like object
-        #: * `status` – an :class:`EntryUpdateStatus` value
-        #:
-        #: Each function should return :const:`None`.
-        #:
-        #: .. warning::
-        #:
-        #:  The only `entry` attributes guaranteed to be present are
-        #:  :attr:`~Entry.feed_url`, :attr:`~Entry.id`,
-        #:  and :attr:`~Entry.resource_id`;
-        #:  all other attributes may be missing
-        #:  (accessing them may raise :exc:`AttributeError`).
-        #:
-        #: .. versionadded:: 1.20
-        #:
-        self.after_entry_update_hooks: MutableSequence[AfterEntryUpdateHook] = []
-
-        #: List of functions called for each updated feed
-        #: before the feed is updated.
-        #:
-        #: Each function is called with:
-        #:
-        #: * `reader` – the :class:`Reader` instance
-        #: * `feed` – the :class:`str` feed URL
-        #:
-        #: Each function should return :const:`None`.
-        #:
-        #: .. versionadded:: 2.7
-        #:
-        self.before_feed_update_hooks: MutableSequence[FeedUpdateHook] = []
-
-        #: List of functions called for each updated feed
-        #: after the feed is updated.
-        #:
-        #: Each function is called with:
-        #:
-        #: * `reader` – the :class:`Reader` instance
-        #: * `feed` – the :class:`str` feed URL
-        #:
-        #: Each function should return :const:`None`.
-        #:
-        #: .. versionadded:: 2.2
-        #:
-        self.after_feed_update_hooks: MutableSequence[FeedUpdateHook] = []
-
-        #: List of functions called *once* before updating any feeds,
-        #: at the beginning of :meth:`update_feeds` / :meth:`update_feeds_iter`,
-        #: but not :meth:`update_feed`.
-        #:
-        #: Each function is called with:
-        #:
-        #: * `reader` – the :class:`Reader` instance
-        #:
-        #: Each function should return :const:`None`.
-        #:
-        #: .. versionadded:: 2.12
-        #:
-        self.before_feeds_update_hooks: MutableSequence[FeedsUpdateHook] = []
-
-        #: List of functions called *once* after updating all feeds,
-        #: at the end of :meth:`update_feeds` / :meth:`update_feeds_iter`,
-        #: but not :meth:`update_feed`.
-        #:
-        #: Each function is called with:
-        #:
-        #: * `reader` – the :class:`Reader` instance
-        #:
-        #: Each function should return :const:`None`.
-        #:
-        #: .. versionadded:: 2.12
-        #:
-        self.after_feeds_update_hooks: MutableSequence[FeedsUpdateHook] = []
+        self._update_hooks = UpdateHooks(self)
 
         if _called_directly:
             warnings.warn(
                 "Reader objects should be created using make_reader(); the Reader "
                 "constructor is not stable yet and may change without any notice.",
+                stacklevel=2,
             )
 
-    def __enter__(self: _TReader) -> _TReader:
+    def __enter__(self) -> Self:
         self._storage.__enter__()
         return self
 
@@ -617,64 +550,17 @@ class Reader:
         broken: bool | None = None,
         updates_enabled: bool | None = None,
         new: bool | None = None,
-        sort: FeedSortOrder = 'title',
+        sort: FeedSort = 'title',
         limit: int | None = None,
         starting_after: FeedInput | None = None,
     ) -> Iterable[Feed]:
         """Get all or some of the feeds.
 
-        The ``tags`` argument can be a list of one or more feed tags.
-        Multiple tags are interpreted as a conjunction (AND).
-        To use a disjunction (OR), use a nested list.
-        To negate a tag, prefix the tag value with a minus sign (``-``).
-        Examples:
-
-        ``['one']``
-
-            one
-
-        ``['one', 'two']``
-        ``[['one'], ['two']]``
-
-            one AND two
-
-        ``[['one', 'two']]``
-
-            one OR two
-
-        ``[['one', 'two'], 'three']``
-
-            (one OR two) AND three
-
-        ``['one', '-two']``
-
-            one AND NOT two
-
-        Special values :const:`True` and :const:`False`
-        match feeds with any tags and no tags, respectively.
-
-        ``True``
-        ``[True]``
-
-            *any tags*
-
-        ``False``
-        ``[False]``
-
-            *no tags*
-
-        ``[True, '-one']``
-
-            *any tags* AND NOT one
-
-        ``[[False, 'one']]``
-
-            *no tags* OR one
-
         Args:
             feed (str or tuple(str) or Feed or None): Only return the feed with this URL.
             tags (None or bool or list(str or bool or list(str or bool))):
-                Only return feeds matching these tags.
+                Only return feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             broken (bool or None): Only return broken / healthy feeds.
             updates_enabled (bool or None):
                 Only return feeds that have updates enabled / disabled.
@@ -712,9 +598,7 @@ class Reader:
             The ``new`` keyword argument.
 
         """
-        filter_options = FeedFilterOptions.from_args(
-            feed, tags, broken, updates_enabled, new
-        )
+        filter = FeedFilter.from_args(feed, tags, broken, updates_enabled, new)
 
         if sort not in ('title', 'added'):
             raise ValueError("sort should be one of ('title', 'added')")
@@ -722,16 +606,9 @@ class Reader:
         if limit is not None:
             if not isinstance(limit, numbers.Integral) or limit < 1:
                 raise ValueError("limit should be a positive integer")
+        starting_after = _feed_argument(starting_after) if starting_after else None
 
-        rv = self._storage.get_feeds(
-            filter_options,
-            sort,
-            limit,
-            _feed_argument(starting_after) if starting_after else None,
-        )
-
-        for rv_feed in rv:
-            yield fix_datetime_tzinfo(rv_feed, 'updated', 'added', 'last_updated')
+        return self._storage.get_feeds(filter, sort, limit, starting_after)
 
     @overload
     def get_feed(self, feed: FeedInput, /) -> Feed:  # pragma: no cover
@@ -789,12 +666,11 @@ class Reader:
     ) -> FeedCounts:
         """Count all or some of the feeds.
 
-        See :meth:`~Reader.get_feeds()` for details on how filtering works.
-
         Args:
             feed (str or tuple(str) or Feed or None): Only count the feed with this URL.
             tags (None or bool or list(str or bool or list(str or bool))):
-                Only count feeds matching these tags.
+                Only count feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             broken (bool or None): Only count broken / healthy feeds.
             updates_enabled (bool or None):
                 Only count feeds that have updates enabled / disabled.
@@ -814,10 +690,8 @@ class Reader:
             The ``new`` keyword argument.
 
         """
-        filter_options = FeedFilterOptions.from_args(
-            feed, tags, broken, updates_enabled, new
-        )
-        return self._storage.get_feed_counts(filter_options)
+        filter = FeedFilter.from_args(feed, tags, broken, updates_enabled, new)
+        return self._storage.get_feed_counts(filter)
 
     def set_feed_user_title(self, feed: FeedInput, title: str | None, /) -> None:
         """Set a user-defined title for a feed.
@@ -889,10 +763,16 @@ class Reader:
         new: bool | None = None,
         workers: int = 1,
     ) -> None:
-        """Update all or some of the feeds.
+        r"""Update all or some of the feeds.
 
         Silently skip feeds that raise :exc:`ParseError`.
 
+        Re-raise :attr:`before_feeds_update_hooks` failures immediately.
+        Collect all other update hook failures
+        and re-raise them as an :exc:`UpdateHookErrorGroup`;
+        currently, only the exceptions for the first 5 feeds
+        with hook failures are collected.
+
         By default, update all the feeds that have updates enabled.
 
         Roughly equivalent to ``for _ in reader.update_feeds_iter(...): pass``.
@@ -900,7 +780,8 @@ class Reader:
         Args:
             feed (str or tuple(str) or Feed or None): Only update the feed with this URL.
             tags (None or bool or list(str or bool or list(str or bool))):
-                Only update feeds matching these tags.
+                Only update feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             broken (bool or None): Only update broken / healthy feeds.
             updates_enabled (bool or None):
                 Only update feeds that have updates enabled / disabled.
@@ -911,6 +792,8 @@ class Reader:
             workers (int): Number of threads to use when getting the feeds.
 
         Raises:
+            UpdateHookError: For unexpected hook exceptions.
+            UpdateError
             StorageError
 
         .. versionchanged:: 1.11
@@ -938,28 +821,54 @@ class Reader:
             The ``feed``, ``tags``, ``broken``, and ``updates_enabled``
             keyword arguments.
 
+        .. versionchanged:: 3.8
+            Wrap unexpected update hook exceptions in :exc:`UpdateHookError`.
+            Try to update all the feeds, don’t stop after a feed/entry hook fails.
+
+        .. versionchanged:: 3.8
+            Document this method can raise non-feed-related :exc:`UpdateError`\s
+            (other than :exc:`UpdateHookError`).
+
         """
-        results = self.update_feeds_iter(
-            feed=feed,
-            tags=tags,
-            broken=broken,
-            updates_enabled=updates_enabled,
-            new=new,
-            workers=workers,
-        )
+        hook_errors = self._update_hooks.group("some hooks failed")
+        try:
+            results = self.update_feeds_iter(
+                feed=feed,
+                tags=tags,
+                broken=broken,
+                updates_enabled=updates_enabled,
+                new=new,
+                workers=workers,
+            )
 
-        for url, value in results:
-            if isinstance(value, ParseError):
-                log.exception(
-                    "update feed %r: error while getting/parsing feed, "
-                    "skipping; exception: %r",
-                    url,
-                    value.__cause__,
-                    exc_info=value,
-                )
-                continue
+            for url, value in results:
+                if isinstance(value, ParseError):
+                    log.exception(
+                        "update feed %r: error while getting/parsing feed, "
+                        "skipping; exception: %r",
+                        url,
+                        value.__cause__,
+                        exc_info=value,
+                    )
+                    continue
 
-            assert not isinstance(value, Exception), value
+                if isinstance(value, UpdateHookError):
+                    hook_errors.add(value, url, limit=5)
+                    continue
+
+                assert not isinstance(value, Exception), value
+
+        except SingleUpdateHookError as e:
+            assert e.when == 'before_feeds_update', e
+            raise
+
+        except UpdateHookErrorGroup as e:
+            for exc in e.exceptions:
+                assert isinstance(exc, SingleUpdateHookError), exc
+                assert exc.when == 'after_feeds_update', exc
+            hook_errors.add(e)
+
+        hook_errors.close()
 
     def update_feeds_iter(
         self,
@@ -972,16 +881,23 @@ class Reader:
         workers: int = 1,
         _call_feeds_update_hooks: bool = True,
     ) -> Iterable[UpdateResult]:
-        """Update all or some of the feeds.
+        r"""Update all or some of the feeds.
 
         Yield information about each updated feed.
 
+        Re-raise :attr:`before_feeds_update_hooks` failures immediately.
+        Yield feed/entry update hook failures.
+        Collect :attr:`after_feeds_update_hooks` failures
+        and re-raise them as an :exc:`UpdateHookErrorGroup`
+        after updating all the feeds.
+
         By default, update all the feeds that have updates enabled.
 
         Args:
             feed (str or tuple(str) or Feed or None): Only update the feed with this URL.
             tags (None or bool or list(str or bool or list(str or bool))):
-                Only update feeds matching these tags.
+                Only update feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             broken (bool or None): Only update broken / healthy feeds.
             updates_enabled (bool or None):
                 Only update feeds that have updates enabled / disabled.
@@ -1000,11 +916,22 @@ class Reader:
                   since the last update
                 * an exception instance
 
-                Currently, the exception is always a :exc:`ParseError`,
-                but other :exc:`ReaderError` subclasses may be yielded
+                Currently, the exception can be:
+
+                * :exc:`ParseError`, if retrieving/parsing the feed failed
+                * :exc:`UpdateHookError`, for unexpected hook exceptions
+                  raised in :attr:`before_feed_update_hooks`,
+                  :attr:`after_entry_update_hooks`, or
+                  :attr:`after_feed_update_hooks`
+
+                ...but other :exc:`UpdateError` subclasses may be yielded
                 in the future.
 
         Raises:
+            UpdateHookError: For unexpected hook exceptions raised in
+                :attr:`before_feeds_update_hooks` or
+                :attr:`after_feeds_update_hooks`.
+            UpdateError: For non-feed-related update exceptions.
             StorageError
 
         .. versionadded:: 1.14
@@ -1023,34 +950,44 @@ class Reader:
             The ``feed``, ``tags``, ``broken``, and ``updates_enabled``
             keyword arguments.
 
+        .. versionchanged:: 3.8
+            Wrap unexpected update hook exceptions in :exc:`UpdateHookError`.
+            Try to update all the feeds, don’t stop after a feed/entry hook fails.
+
+        .. versionchanged:: 3.8
+            Document this method can raise non-feed-related :exc:`UpdateError`\s
+            (other than :exc:`UpdateHookError`).
+
         """
-        filter_options = FeedFilterOptions.from_args(
-            feed, tags, broken, updates_enabled, new
-        )
+        filter = FeedFilter.from_args(feed, tags, broken, updates_enabled, new)
 
         if workers < 1:
             raise ValueError("workers must be a positive integer")
 
-        make_map = nullcontext(builtins.map) if workers == 1 else make_pool_map(workers)
+        make_map: MapContextManager[Any, Any] = (
+            nullcontext(builtins.map) if workers == 1 else make_pool_map(workers)
+        )
 
         if _call_feeds_update_hooks:
-            for hook in self.before_feeds_update_hooks:
-                hook(self)
+            self._update_hooks.run('before_feeds_update', None)
 
         with make_map as map:
-            yield from Pipeline.from_reader(self, map).update(filter_options)
+            yield from Pipeline.from_reader(self, map).update(filter)
 
         if _call_feeds_update_hooks:
-            for hook in self.after_feeds_update_hooks:
-                hook(self)
+            hook_errors = self._update_hooks.group(
+                "got unexpected after-update hook errors"
+            )
+            hook_errors.run('after_feeds_update', None)
+            hook_errors.close()
 
     def update_feed(self, feed: FeedInput, /) -> UpdatedFeed | None:
-        """Update a single feed.
+        r"""Update a single feed.
 
         The feed will be updated even if updates are disabled for it.
 
         Like ``next(iter(reader.update_feeds_iter(feed=feed, updates_enabled=None)))[1]``,
-        but raises the :exc:`ParseError`, if any.
+        but raises the :exc:`UpdateError`, if any.
 
         Args:
             feed (str or tuple(str) or Feed): The feed URL.
@@ -1064,6 +1001,8 @@ class Reader:
         Raises:
             FeedNotFoundError
             ParseError
+            UpdateHookError: For unexpected hook exceptions.
+            UpdateError
             StorageError
 
         .. versionchanged:: 1.14
@@ -1076,6 +1015,13 @@ class Reader:
         .. versionchanged:: 3.0
             The ``feed`` argument is now positional-only.
 
+        .. versionchanged:: 3.8
+            Wrap unexpected update hook exceptions in :exc:`UpdateHookError`.
+
+        .. versionchanged:: 3.8
+            Document this method can raise :exc:`UpdateError`\s
+            (other than :exc:`ParseError` and :exc:`UpdateHookError`).
+
         """
         _, rv = zero_or_one(
             self.update_feeds_iter(
@@ -1089,7 +1035,7 @@ class Reader:
 
     @staticmethod
     def _now() -> datetime:
-        return datetime.utcnow()
+        return datetime.now(timezone.utc)
 
     def get_entries(
         self,
@@ -1099,8 +1045,9 @@ class Reader:
         read: bool | None = None,
         important: TristateFilterInput = None,
         has_enclosures: bool | None = None,
+        tags: TagFilterInput = None,
         feed_tags: TagFilterInput = None,
-        sort: EntrySortOrder = 'recent',
+        sort: EntrySort = 'recent',
         limit: int | None = None,
         starting_after: EntryInput | None = None,
     ) -> Iterable[Entry]:
@@ -1148,9 +1095,12 @@ class Reader:
                 :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only return entries that (don't)
                 have enclosures.
+            tags (None or bool or list(str or bool or list(str or bool))):
+                Only return entries matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
-                Only return the entries from feeds matching these tags;
-                works like the :meth:`~Reader.get_feeds()` ``tags`` argument.
+                Only return entries from feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             sort (str): How to order entries; one of ``'recent'`` (default)
                 or ``'random'``.
             limit (int or None): A limit on the number of entries to be returned;
@@ -1178,13 +1128,16 @@ class Reader:
         .. versionchanged:: 3.5
             The ``important`` argument also accepts string values.
 
+        .. versionadded:: 3.11
+            The ``tags`` keyword argument.
+
         """
 
         # If we ever implement pagination, consider following the guidance in
         # https://specs.openstack.org/openstack/api-wg/guidelines/pagination_filter_sort.html
 
-        filter_options = EntryFilterOptions.from_args(
-            feed, entry, read, important, has_enclosures, feed_tags
+        filter = EntryFilter.from_args(
+            feed, entry, read, important, has_enclosures, tags, feed_tags
         )
 
         if sort not in ('recent', 'random'):
@@ -1193,32 +1146,12 @@ class Reader:
         if limit is not None:
             if not isinstance(limit, numbers.Integral) or limit < 1:
                 raise ValueError("limit should be a positive integer")
+        starting_after = _entry_argument(starting_after) if starting_after else None
 
         if starting_after and sort == 'random':
             raise ValueError("using starting_after with sort='random' not supported")
 
-        now = self._now()
-        rv = self._storage.get_entries(
-            now,
-            filter_options,
-            sort,
-            limit,
-            _entry_argument(starting_after) if starting_after else None,
-        )
-
-        for rv_entry in rv:
-            yield fix_datetime_tzinfo(
-                rv_entry,
-                'updated',
-                'published',
-                'added',
-                'last_updated',
-                'read_modified',
-                'important_modified',
-                feed=fix_datetime_tzinfo(
-                    rv_entry.feed, 'updated', 'added', 'last_updated'
-                ),
-            )
+        return self._storage.get_entries(filter, sort, limit, starting_after)
 
     @overload
     def get_entry(self, entry: EntryInput, /) -> Entry:  # pragma: no cover
@@ -1273,12 +1206,11 @@ class Reader:
         read: bool | None = None,
         important: TristateFilterInput = None,
         has_enclosures: bool | None = None,
+        tags: TagFilterInput = None,
         feed_tags: TagFilterInput = None,
     ) -> EntryCounts:
         """Count all or some of the entries.
 
-        See :meth:`~Reader.get_entries()` for details on how filtering works.
-
         Args:
             feed (str or tuple(str) or Feed or None): Only count the entries for this feed.
             entry (tuple(str, str) or Entry or None):
@@ -1290,8 +1222,12 @@ class Reader:
                 :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only count entries that (don't)
                 have enclosures.
+            tags (None or bool or list(str or bool or list(str or bool))):
+                Only count entries matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
-                Only count the entries from feeds matching these tags.
+                Only count entries from feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
 
         Returns:
             EntryCounts:
@@ -1304,13 +1240,16 @@ class Reader:
         .. versionchanged:: 3.5
             The ``important`` argument also accepts string values.
 
+        .. versionadded:: 3.11
+            The ``tags`` keyword argument.
+
         """
 
-        filter_options = EntryFilterOptions.from_args(
-            feed, entry, read, important, has_enclosures, feed_tags
+        filter = EntryFilter.from_args(
+            feed, entry, read, important, has_enclosures, tags, feed_tags
         )
         now = self._now()
-        return self._storage.get_entry_counts(now, filter_options)
+        return self._storage.get_entry_counts(now, filter)
 
     def set_entry_read(
         self,
@@ -1349,16 +1288,15 @@ class Reader:
         if read not in (True, False):
             raise ValueError("read should be one of (True, False)")
 
-        modified_naive: datetime | None
+        modified_aware: datetime | None
         if isinstance(modified, MissingType):
-            modified_naive = self._now()
+            modified_aware = self._now()
         elif modified is None:
-            modified_naive = None
+            modified_aware = None
         else:
-            modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
+            modified_aware = modified.astimezone(timezone.utc)
 
-        feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_read(feed_url, entry_id, read, modified_naive)
+        self._storage.set_entry_read(_entry_argument(entry), read, modified_aware)
 
     def mark_entry_as_read(self, entry: EntryInput, /) -> None:
         """Mark an entry as read.
@@ -1442,16 +1380,17 @@ class Reader:
         if important not in (True, False, None):
             raise ValueError("important should be one of (True, False, None)")
 
-        modified_naive: datetime | None
+        modified_aware: datetime | None
         if isinstance(modified, MissingType):
-            modified_naive = self._now()
+            modified_aware = self._now()
         elif modified is None:
-            modified_naive = None
+            modified_aware = None
         else:
-            modified_naive = modified.astimezone(timezone.utc).replace(tzinfo=None)
+            modified_aware = modified.astimezone(timezone.utc)
 
-        feed_url, entry_id = _entry_argument(entry)
-        self._storage.mark_as_important(feed_url, entry_id, important, modified_naive)
+        self._storage.set_entry_important(
+            _entry_argument(entry), important, modified_aware
+        )
 
     def mark_entry_as_important(self, entry: EntryInput, /) -> None:
         """Mark an entry as important.
@@ -1668,6 +1607,7 @@ class Reader:
         read: bool | None = None,
         important: TristateFilterInput = None,
         has_enclosures: bool | None = None,
+        tags: TagFilterInput = None,
         feed_tags: TagFilterInput = None,
         sort: SearchSortOrder = 'relevant',
         limit: int | None = None,
@@ -1739,9 +1679,12 @@ class Reader:
                 :data:`~reader.types.TristateFilterInput` string filters.
             has_enclosures (bool or None): Only search entries that (don't)
                 have enclosures.
+            tags (None or bool or list(str or bool or list(str or bool))):
+                Only search entries matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
-                Only return the entries from feeds matching these tags;
-                works like the :meth:`~Reader.get_feeds()` ``tags`` argument.
+                Only search entries from feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             sort (str): How to order results; one of ``'relevant'`` (default),
                 ``'recent'``, or ``'random'``.
             limit (int or None): A limit on the number of results to be returned;
@@ -1775,9 +1718,12 @@ class Reader:
         .. versionchanged:: 3.5
             The ``important`` argument also accepts string values.
 
+        .. versionadded:: 3.11
+            The ``tags`` keyword argument.
+
         """
-        filter_options = EntryFilterOptions.from_args(
-            feed, entry, read, important, has_enclosures, feed_tags
+        filter = EntryFilter.from_args(
+            feed, entry, read, important, has_enclosures, tags, feed_tags
         )
 
         if sort not in ('relevant', 'recent', 'random'):
@@ -1786,19 +1732,12 @@ class Reader:
         if limit is not None:
             if not isinstance(limit, numbers.Integral) or limit < 1:
                 raise ValueError("limit should be a positive integer")
+        starting_after = _entry_argument(starting_after) if starting_after else None
 
         if starting_after and sort == 'random':
             raise ValueError("using starting_after with sort='random' not supported")
 
-        now = self._now()
-        return self._search.search_entries(
-            query,
-            now,
-            filter_options,
-            sort,
-            limit,
-            _entry_argument(starting_after) if starting_after else None,
-        )
+        return self._search.search_entries(query, filter, sort, limit, starting_after)
 
     def search_entry_counts(
         self,
@@ -1810,12 +1749,12 @@ class Reader:
         read: bool | None = None,
         important: TristateFilterInput = None,
         has_enclosures: bool | None = None,
+        tags: TagFilterInput = None,
         feed_tags: TagFilterInput = None,
     ) -> EntrySearchCounts:
         """Count entries matching a full-text search query.
 
-        See :meth:`~Reader.search_entries()` for details on how
-        the query syntax and filtering work.
+        See :meth:`~Reader.search_entries()` for details on the query syntax.
 
         Search must be enabled to call this method.
 
@@ -1831,8 +1770,12 @@ class Reader:
             important (bool or None): Only count (un)important entries.
             has_enclosures (bool or None): Only count entries that (don't)
                 have enclosures.
+            tags (None or bool or list(str or bool or list(str or bool))):
+                Only count entries matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
             feed_tags (None or bool or list(str or bool or list(str or bool))):
-                Only count the entries from feeds matching these tags.
+                Only count entries from feeds matching these tags;
+                see :data:`~reader.types.TagFilterInput` for details.
 
         Returns:
             EntrySearchCounts:
@@ -1851,13 +1794,16 @@ class Reader:
         .. versionchanged:: 3.5
             The ``important`` argument also accepts string values.
 
+        .. versionadded:: 3.11
+            The ``tags`` keyword argument.
+
         """
 
-        filter_options = EntryFilterOptions.from_args(
-            feed, entry, read, important, has_enclosures, feed_tags
+        filter = EntryFilter.from_args(
+            feed, entry, read, important, has_enclosures, tags, feed_tags
         )
         now = self._now()
-        return self._search.search_entry_counts(query, now, filter_options)
+        return self._search.search_entry_counts(query, now, filter)
 
     def get_tags(
         self, resource: ResourceInput, /, *, key: str | None = None
@@ -2198,3 +2144,140 @@ class Reader:
             self._reserved_name_scheme = NameScheme.from_value(value)
         except Exception as e:
             raise AttributeError(f"invalid reserved name scheme: {value}") from e
+
+    @property
+    def before_feeds_update_hooks(self) -> MutableSequence[FeedsUpdateHook]:
+        """List of functions called *once* before updating any feeds,
+        at the beginning of :meth:`update_feeds` / :meth:`update_feeds_iter`,
+        but not :meth:`update_feed`.
+
+        Each function is called with:
+
+        * `reader` – the :class:`Reader` instance
+
+        Each function should return :const:`None`.
+
+        The hooks are run in order.
+        Exceptions raised by hooks are wrapped in a :exc:`SingleUpdateHookError`
+        and re-raised (hooks after the one that failed are not run).
+
+        .. versionadded:: 2.12
+
+        .. versionchanged:: 3.8
+            Wrap unexpected exceptions in :exc:`UpdateHookError`.
+
+        """
+        return self._update_hooks['before_feeds_update']
+
+    @property
+    def before_feed_update_hooks(self) -> MutableSequence[FeedUpdateHook]:
+        """List of functions called for each updated feed
+        before the feed is updated.
+
+        Each function is called with:
+
+        * `reader` – the :class:`Reader` instance
+        * `feed` – the :class:`str` feed URL
+
+        Each function should return :const:`None`.
+
+        The hooks are run in order.
+        Exceptions raised by hooks are wrapped in a :exc:`SingleUpdateHookError`
+        and re-raised (hooks after the one that failed are not run).
+
+        .. versionadded:: 2.7
+
+        .. versionchanged:: 3.8
+            Wrap unexpected exceptions in :exc:`UpdateHookError`.
+
+        """
+        return self._update_hooks['before_feed_update']
+
+    @property
+    def after_entry_update_hooks(self) -> MutableSequence[AfterEntryUpdateHook]:
+        """List of functions called for each updated entry
+        after the feed is updated.
+
+        Each function is called with:
+
+        * `reader` – the :class:`Reader` instance
+        * `entry` – an :class:`Entry`-like object
+        * `status` – an :class:`EntryUpdateStatus` value
+
+        Each function should return :const:`None`.
+
+        .. warning::
+
+            The only `entry` attributes guaranteed to be present are
+            :attr:`~Entry.feed_url`, :attr:`~Entry.id`,
+            and :attr:`~Entry.resource_id`;
+            all other attributes may be missing
+            (accessing them may raise :exc:`AttributeError`).
+
+        The hooks are run in order.
+        Exceptions raised by hooks are wrapped in a :exc:`SingleUpdateHookError`,
+        collected, and re-raised as an :exc:`UpdateHookErrorGroup`
+        after all the hooks are run;
+        currently, only the exceptions for the first 5 entries
+        with hook failures are collected.
+
+        .. versionadded:: 1.20
+
+        .. versionchanged:: 3.8
+            Wrap unexpected exceptions in :exc:`UpdateHookError`.
+            Try to run all hooks, don't stop after one fails.
+
+        """
+        return self._update_hooks['after_entry_update']
+
+    @property
+    def after_feed_update_hooks(self) -> MutableSequence[FeedUpdateHook]:
+        """List of functions called for each updated feed
+        after the feed is updated.
+
+        Each function is called with:
+
+        * `reader` – the :class:`Reader` instance
+        * `feed` – the :class:`str` feed URL
+
+        Each function should return :const:`None`.
+
+        The hooks are run in order.
+        Exceptions raised by hooks are wrapped in a :exc:`SingleUpdateHookError`,
+        collected, and re-raised as an :exc:`UpdateHookErrorGroup`
+        after all the hooks are run.
+
+        .. versionadded:: 2.2
+
+        .. versionchanged:: 3.8
+            Wrap unexpected exceptions in :exc:`UpdateHookError`.
+            Try to run all hooks, don't stop after one fails.
+
+        """
+        return self._update_hooks['after_feed_update']
+
+    @property
+    def after_feeds_update_hooks(self) -> MutableSequence[FeedsUpdateHook]:
+        """List of functions called *once* after updating all feeds,
+        at the end of :meth:`update_feeds` / :meth:`update_feeds_iter`,
+        but not :meth:`update_feed`.
+
+        Each function is called with:
+
+        * `reader` – the :class:`Reader` instance
+
+        Each function should return :const:`None`.
+
+        The hooks are run in order.
+        Exceptions raised by hooks are wrapped in a :exc:`SingleUpdateHookError`,
+        collected, and re-raised as an :exc:`UpdateHookErrorGroup`
+        after all the hooks are run.
+
+        .. versionadded:: 2.12
+
+        .. versionchanged:: 3.8
+            Wrap unexpected exceptions in :exc:`UpdateHookError`.
+            Try to run all hooks, don't stop after one fails.
+
+        """
+        return self._update_hooks['after_feeds_update']
