@@ -4,9 +4,9 @@ Built-in plug-ins.
 """
 from __future__ import annotations
 
+from collections.abc import Callable
 from collections.abc import Iterable
 from pkgutil import resolve_name
-from typing import Callable
 from typing import TYPE_CHECKING
 from typing import Union
 
