@@ -4,6 +4,7 @@ import dataclasses
 import enum
 import re
 import traceback
+from collections.abc import Callable
 from collections.abc import Iterable
 from collections.abc import Mapping
 from collections.abc import Sequence
@@ -12,25 +13,23 @@ from dataclasses import field
 from datetime import datetime
 from types import MappingProxyType
 from typing import Any
-from typing import Callable
 from typing import cast
 from typing import Literal
 from typing import NamedTuple
 from typing import overload
 from typing import Protocol
-from typing import runtime_checkable
-from typing import TypeVar
+from typing import TYPE_CHECKING
 from typing import Union
 
-from reader.exceptions import ReaderError
+from reader.exceptions import UpdateError
 
 # noreorder
 # can't be defined here because of circular imports
 from reader._utils import MISSING as MISSING  # noqa: F401
 from reader._utils import MissingType as MissingType  # noqa: F401
 
-
-_T = TypeVar('_T')
+if TYPE_CHECKING:  # pragma: no cover
+    from typing_extensions import Self
 
 
 class _namedtuple_compat:
@@ -39,17 +38,7 @@ class _namedtuple_compat:
 
     # TODO: can we get rid of _namedtuple_compat?
 
-    @classmethod
-    def _make(cls: type[_T], iterable: Iterable[Any]) -> _T:
-        iterable = tuple(iterable)
-        attrs_len = len(dataclasses.fields(cls))  # type: ignore[arg-type]
-        if len(iterable) != attrs_len:
-            raise TypeError(
-                'Expected %d arguments, got %d' % (attrs_len, len(iterable))
-            )
-        return cls(*iterable)
-
-    def _replace(self: _T, **kargs: Any) -> _T:
+    def _replace(self, **kargs: Any) -> Self:
         return dataclasses.replace(self, **kargs)  # type: ignore[type-var]
 
     def _asdict(self) -> dict[str, Any]:
@@ -137,9 +126,14 @@ class Feed(_namedtuple_compat):
     #: .. versionadded:: 1.3
     last_updated: datetime | None = None
 
-    #: If a :exc:`ParseError` happend during the last update, its cause.
+    #: If a :exc:`UpdateError` happened during the last update, its details.
     #:
     #: .. versionadded:: 1.3
+    #:
+    #: .. versionchanged:: 3.9
+    #:  Store the details of any :exc:`UpdateError` (except hook errors),
+    #:  not just the ``__cause__`` of :exc:`ParseError`\s.
+    #:
     last_exception: ExceptionInfo | None = None
 
     #: Whether updates are enabled for this feed.
@@ -157,9 +151,6 @@ class Feed(_namedtuple_compat):
         return (self.url,)
 
 
-_EI = TypeVar('_EI', bound='ExceptionInfo')
-
-
 @dataclass(frozen=True)
 class ExceptionInfo(_namedtuple_compat):
 
@@ -182,7 +173,7 @@ class ExceptionInfo(_namedtuple_compat):
     traceback_str: str
 
     @classmethod
-    def from_exception(cls: type[_EI], exc: BaseException) -> _EI:
+    def from_exception(cls, exc: BaseException) -> Self:
         return cls(
             f'{type(exc).__module__}.{type(exc).__qualname__}',
             str(exc),
@@ -305,6 +296,16 @@ class Entry(_namedtuple_compat):
     #: .. versionadded:: 1.8
     original_feed_url: str = cast(str, None)
 
+    #: Change sequence.
+    #:
+    #: May be :const:`None` when change tracking is disabled.
+    #:
+    #: .. admonition:: Unstable
+    #:
+    #:  This field is part of the unstable :ref:`change tracking API <changes>`.
+    #:
+    _sequence: bytes | None = None
+
     # feed should not have a default, but I'd prefer objects that aren't
     # entry data to be at the end, and dataclasses don't support keyword-only
     # arguments yet.
@@ -425,9 +426,6 @@ class Enclosure(_namedtuple_compat):
     length: int | None = None
 
 
-_HS = TypeVar('_HS', bound='HighlightedString')
-
-
 @dataclass(frozen=True)
 class HighlightedString:
 
@@ -481,7 +479,7 @@ class HighlightedString:
         return self.value
 
     @classmethod
-    def extract(cls: type[_HS], text: str, before: str, after: str) -> _HS:
+    def extract(cls, text: str, before: str, after: str) -> Self:
         """Extract highlights with before/after markers from text.
 
         >>> HighlightedString.extract( '>one< two', '>', '<')
@@ -667,8 +665,8 @@ class EntryUpdateStatus(enum.Enum):
 # because we sometimes get them dynamically, e.g.
 # https://github.com/lemon24/reader/blob/1.18/src/reader/_app/__init__.py#L151
 #
-FeedSortOrder = Literal['title', 'added']
-EntrySortOrder = Literal['recent', 'random']
+FeedSort = Literal['title', 'added']
+EntrySort = Literal['recent', 'random']
 SearchSortOrder = Literal['relevant', 'recent', 'random']
 
 
@@ -683,9 +681,7 @@ JSONType = Union[dict[str, JSONValue], list[JSONValue]]
 # see https://github.com/python/mypy/issues/8056.
 
 
-@runtime_checkable
 class FeedLike(Protocol):
-
     # We don't use "url: str" because we don't care if url is writable.
 
     @property
@@ -693,7 +689,6 @@ class FeedLike(Protocol):
         ...
 
 
-@runtime_checkable
 class EntryLike(Protocol):
     @property
     def id(self) -> str:  # pragma: no cover
@@ -715,24 +710,26 @@ AnyResourceId = Union[ResourceId, None, tuple[None], tuple[None, None]]
 
 
 def _feed_argument(feed: FeedInput) -> str:
-    if isinstance(feed, FeedLike):
-        rv = feed.url
-    elif isinstance(feed, tuple) and len(feed) == 1:
-        rv = feed[0]
-    else:
-        rv = feed
+    try:
+        rv = feed.url  # type: ignore[union-attr]
+    except AttributeError:
+        if isinstance(feed, tuple) and len(feed) == 1:
+            rv = feed[0]
+        else:
+            rv = feed
     if isinstance(rv, str):
         return rv
     raise ValueError(f'invalid feed argument: {feed!r}')
 
 
 def _entry_argument(entry: EntryInput) -> tuple[str, str]:
-    if isinstance(entry, EntryLike):
-        rv = _feed_argument(entry.feed_url), entry.id
-    elif isinstance(entry, tuple) and len(entry) == 2:
-        rv = entry
-    else:
-        rv = None
+    try:
+        rv = entry.feed_url, entry.id  # type: ignore[union-attr]
+    except AttributeError:
+        if isinstance(entry, tuple) and len(entry) == 2:
+            rv = entry
+        else:
+            rv = None
     if rv:
         feed_url, entry_id = rv
         if isinstance(feed_url, str) and isinstance(entry_id, str):
@@ -771,6 +768,59 @@ def _resource_argument(resource: ResourceInput) -> ResourceId:
 
 # str explicitly excluded, to allow for a string-based query language;
 # https://github.com/lemon24/reader/issues/184#issuecomment-689587006
+
+#: Possible values for filtering resources by their tags.
+#:
+#: Tag filters consist of a list of one or more tags.
+#: Multiple tags are interpreted as a conjunction (AND).
+#: To use a disjunction (OR), use a nested list.
+#: To negate a tag, prefix the tag value with a minus sign (``-``).
+#: Examples:
+#:
+#: ``['one']``
+#:
+#:     one
+#:
+#: ``['one', 'two']``
+#: ``[['one'], ['two']]``
+#:
+#:     one AND two
+#:
+#: ``[['one', 'two']]``
+#:
+#:     one OR two
+#:
+#: ``[['one', 'two'], 'three']``
+#:
+#:     (one OR two) AND three
+#:
+#: ``['one', '-two']``
+#:
+#:     one AND NOT two
+#:
+#: Special values :const:`True` and :const:`False`
+#: match resources with any tags and no tags, respectively.
+#:
+#: ``True``
+#: ``[True]``
+#:
+#:     *any tags*
+#:
+#: ``False``
+#: ``[False]``
+#:
+#:     *no tags*
+#:
+#: ``[True, '-one']``
+#:
+#:     *any tags* AND NOT one
+#:
+#: ``[[False, 'one']]``
+#:
+#:     *no tags* OR one
+#:
+#: .. versionadded:: 3.11
+#:
 TagFilterInput = Union[
     None, bool, Sequence[Union[str, bool, Sequence[Union[str, bool]]]]
 ]
@@ -964,19 +1014,19 @@ class UpdateResult(NamedTuple):
     #:  If the server indicated the feed has not changed
     #:  since the last update without returning any data.
     #:
-    #: :exc:`ReaderError`
+    #: :exc:`UpdateError`
     #:
     #:  If there was an error while updating the feed.
     #:
-    value: UpdatedFeed | None | ReaderError
+    #: .. versionchanged:: 3.8
+    #:  Narrow down the error type from :exc:`ReaderError` to :exc:`UpdateError`.
+    #:
+    value: UpdatedFeed | None | UpdateError
 
-    # The exception type is ReaderError and not ParseError
+    # The exception type is UpdateError and not ParseError
     # to allow suppressing new errors without breaking the API:
     # adding a new type to the union breaks the API,
     # not raising an exception type anymore doesn't.
-    # Currently, storage or plugin-raised exceptions
-    # prevent updates for the following feeds (:issue:`218`),
-    # but that's not necessarily by design.
 
     @property
     def updated_feed(self) -> UpdatedFeed | None:
@@ -988,7 +1038,7 @@ class UpdateResult(NamedTuple):
         return self.value if not isinstance(self.value, Exception) else None
 
     @property
-    def error(self) -> ReaderError | None:
+    def error(self) -> UpdateError | None:
         """The exception, if there was an error, :const:`None` otherwise.
 
         .. versionadded:: 2.1
