@@ -55,12 +55,29 @@ def pytest_runtest_setup(item):
     # see the comments in setup.cfg for details.
     for mark in item.iter_markers(name="requires_lxml"):
         no_lxml = [
-            # currently, we have lxml wheels on all supported platforms;
-            # last in bbe1deee5eee800291b5a0e83fc4ab1cb949445c
+            sys.implementation.name == 'pypy' and sys.version_info[:2] > (3, 9),
         ]
         if any(no_lxml):
             pytest.skip("test requires lxml")
 
+    # getting intermittent Flask-context-related errors on pypy:
+    #   AssertionError: Popped wrong app context.
+    #   RuntimeError: Working outside of request context.
+    for mark in item.iter_markers(name="apptest"):
+        if sys.implementation.name == 'pypy':
+            pytest.skip("flask tests are flaky on pypy")
+
+
+@pytest.fixture(autouse=True, scope="session")
+def no_sqlite3_adapters(request):
+    # bring about the removal of deprecated sqlite3 default adapters
+    # (adapters cannot be disabled per-connection like converters)
+    # https://github.com/lemon24/reader/issues/321
+    # TODO: remove this once sqlite3 default adapters are removed
+    original_adapters = sqlite3.adapters.copy()
+    sqlite3.adapters.clear()
+    request.addfinalizer(lambda: sqlite3.adapters.update(original_adapters))
+
 
 @pytest.fixture
 def make_reader(request):
@@ -85,40 +102,27 @@ def storage():
         yield storage
 
 
-# TODO: move to reader_methods
-# TODO: s/call_update_method/update_feed/
-def call_update_feeds(reader, _):
-    reader.update_feeds()
-
-
-def call_update_feeds_workers(reader, _):
-    reader.update_feeds(workers=2)
-
-
-def call_update_feeds_iter(reader, _):
-    for _ in reader.update_feeds_iter():
-        pass
-
-
-def call_update_feeds_iter_workers(reader, _):
-    for _ in reader.update_feeds_iter(workers=2):
-        pass
-
-
-def call_update_feed(reader, url):
-    reader.update_feed(url)
+def slow(*args, **kwargs):
+    return pytest.param(*args, **kwargs, marks=pytest.mark.slow)
 
 
 @pytest.fixture(
     params=[
-        call_update_feeds,
-        pytest.param(call_update_feeds_workers, marks=pytest.mark.slow),
-        call_update_feeds_iter,
-        pytest.param(call_update_feeds_iter_workers, marks=pytest.mark.slow),
-        call_update_feed,
+        m if 'workers' not in m.__name__ else slow(m)
+        for m in reader_methods.update_feed_methods
     ]
 )
-def call_update_method(request):
+def update_feed(request):
+    return request.param
+
+
+@pytest.fixture(
+    params=[
+        m if 'workers' not in m.__name__ else slow(m)
+        for m in reader_methods.update_feeds_iter_methods
+    ]
+)
+def update_feeds_iter(request):
     return request.param
 
 
@@ -174,32 +178,44 @@ def data_dir(tests_dir):
         Storage.chunk_size,
         # rough result size (order of magnitude)
         1,
-        pytest.param(2, marks=pytest.mark.slow),
-        # unchunked query, likely to be ok
-        pytest.param(0, marks=pytest.mark.slow),
+        slow(2),
     ]
 )
 def chunk_size(request):
     return request.param
 
 
-@pytest.fixture(params=reader_methods.get_entries_methods)
+@pytest.fixture(
+    params=[
+        reader_methods.get_entries_recent,
+        slow(reader_methods.get_entries_random),
+        reader_methods.search_entries_relevant,
+        slow(reader_methods.search_entries_recent),
+        slow(reader_methods.search_entries_random),
+    ]
+)
 def get_entries(request):
     yield request.param
 
 
 @pytest.fixture(
     params=[
-        pytest.param(reader_methods.get_entries, marks=pytest.mark.slow),
+        slow(reader_methods.get_entries),
         reader_methods.get_entries_recent,
-        pytest.param(
-            reader_methods.get_entries_recent_paginated, marks=pytest.mark.slow
-        ),
+        slow(reader_methods.get_entries_recent_paginated),
         reader_methods.search_entries_recent,
-        pytest.param(
-            reader_methods.search_entries_recent_paginated, marks=pytest.mark.slow
-        ),
+        slow(reader_methods.search_entries_recent_paginated),
     ],
 )
 def get_entries_recent(request):
     yield request.param
+
+
+@pytest.fixture(
+    params=[
+        reader_methods.get_entry_counts,
+        reader_methods.search_entry_counts,
+    ]
+)
+def get_entry_counts(request):
+    yield request.param
