@@ -5,7 +5,6 @@ import pytest
 from reader._types import entry_data_from_obj
 from reader._types import EntryData
 from reader._types import FeedData
-from reader._types import fix_datetime_tzinfo
 from reader._types import tag_filter_argument
 from reader._types import tristate_filter_argument
 
@@ -16,7 +15,20 @@ TAG_DATA = [
     ([[False]], [False, [False], [[False]]]),
     ([[True], [False]], [[True, False], [[True], [False]]]),
     ([[True, False]], [[[True, False]]]),
-    ([[(False, 'one')]], [['one'], [['one']], ['one', []], [[], ['one'], []]]),
+    (
+        [[(False, 'one')]],
+        [
+            ['one'],
+            [['one']],
+            ['one', []],
+            [[], ['one'], []],
+            ['one', 'one'],
+            [['one'], ['one']],
+            [['one', 'one']],
+        ],
+    ),
+    ([[(False, 'one')], [(False, 'two')]], [['one', 'two'], [['one'], ['two']]]),
+    ([[(False, 'one'), (False, 'two')]], [[['one', 'two']]]),
     ([[(False, 'one')], [(True, 'two')]], [['one', '-two'], [['one'], ['-two']]]),
     ([[(False, 'one'), (True, 'two')]], [[['one', '-two']]]),
     ([[True], [(False, 'one')]], [[True, 'one'], [True, ['one']], [[True], 'one']]),
@@ -82,11 +94,9 @@ def test_entry_data_from_obj(data_dir, data_file):
     exec(data_dir.joinpath(f'{data_file}.rss.py').read_bytes(), expected)
 
     for i, entry in enumerate(expected['entries']):
-        entry_utc = fix_datetime_tzinfo(entry, 'updated', 'published')
+        assert entry == entry_data_from_obj(entry), i
 
-        assert entry == entry_data_from_obj(entry_utc), i
-
-        entry_dict = entry_utc._asdict()
+        entry_dict = entry._asdict()
         if 'content' in entry_dict:
             entry_dict['content'] = [c._asdict() for c in entry_dict['content']]
         if 'enclosures' in entry_dict:
@@ -153,7 +163,6 @@ def test_entry_data_from_obj(data_dir, data_file):
     ],
 )
 def test_entry_data_from_obj_errors(exc, entry):
-
     with pytest.raises(exc):
         entry_data_from_obj(entry)
 
