@@ -9,22 +9,22 @@ import requests
 from utils import make_url_base
 
 from reader import Feed
-from reader._feedparser import FeedparserParser
-from reader._jsonfeed import JSONFeedParser
 from reader._parser import default_parser
 from reader._parser import FeedArgumentTuple
 from reader._parser import Parser
 from reader._parser import RetrieveResult
-from reader._requests_utils import SessionWrapper
-from reader._retrievers import FileRetriever
+from reader._parser.feedparser import FeedparserParser
+from reader._parser.file import FileRetriever
+from reader._parser.jsonfeed import JSONFeedParser
+from reader._parser.requests import SessionWrapper
 from reader._types import FeedData
 from reader._vendor import feedparser
 from reader.exceptions import ParseError
 
 
-@pytest.fixture
-def parse():
-    parse = default_parser('')
+@pytest.fixture(params=[True, False])
+def parse(request):
+    parse = default_parser('', _lazy=request.param)
     yield parse
 
 
@@ -429,7 +429,7 @@ def test_parse_requests_exception(monkeypatch, exc_cls):
         def get(self, *args, **kwargs):
             raise exc
 
-    monkeypatch.setattr('reader._requests_utils.SessionFactory.__call__', BadWrapper)
+    monkeypatch.setattr('reader._parser.requests.SessionFactory.__call__', BadWrapper)
 
     with pytest.raises(ParseError) as excinfo:
         default_parser('')('http://example.com')
@@ -497,7 +497,7 @@ def test_feedparser_parse_call(monkeypatch, parse, make_url, data_dir, exc_cls):
         parse(feed_url)
     assert excinfo.value.__cause__ is exc
     assert excinfo.value.url == feed_url
-    assert 'while reading feed' in excinfo.value.message
+    assert 'error during parser' in excinfo.value.message
 
     assert feedparser_parse.kwargs['resolve_relative_uris'] == True
     assert feedparser_parse.kwargs['sanitize_html'] == True
@@ -784,7 +784,10 @@ def test_feed_root_nonempty(data_dir, scheme):
 
 
 # os_name, root
-RELATIVE_ROOTS = [('nt', 'C:feeds'), ('nt', '\\feeds'),] + [
+RELATIVE_ROOTS = [
+    ('nt', 'C:feeds'),
+    ('nt', '\\feeds'),
+] + [
     (os_name, root)
     for os_name in ['nt', 'posix']
     for root in ['feeds', './feeds', '../feeds']
@@ -875,18 +878,14 @@ def test_normalize_url_errors(monkeypatch, reload_module, os_name, url, reason):
     monkeypatch.setattr('os.name', os_name)
     monkeypatch.setattr('os.path', {'nt': ntpath, 'posix': posixpath}[os_name])
 
-    import reader._url_utils
-    import reader._retrievers
+    import urllib.request
 
-    # reader._url_utils.url2pathname differs based on os.name
-    reload_module(reader._url_utils)
-    reload_module(reader._retrievers)
-
-    import reader._retrievers
+    # urllib.request differs based on os.name
+    reload_module(urllib.request)
 
     with pytest.raises(ValueError) as excinfo:
         try:
-            reader._retrievers.FileRetriever(data_dir)._normalize_url(url)
+            FileRetriever(data_dir)._normalize_url(url)
         finally:
             # pytest.raises() doesn't interact well with our monkeypatching
             reload_module.undo()
@@ -928,7 +927,7 @@ def make_dummy_retriever(name, mime_type='type/subtype', headers=None):
 def make_dummy_parser(prefix='', http_accept=None):
     def parser(url, file, headers):
         parser.last_headers = headers
-        return prefix + file, url
+        return prefix + file, [url]
 
     if http_accept:
         parser.http_accept = http_accept
@@ -951,7 +950,7 @@ def test_parser_selection():
     parse.mount_parser_by_mime_type(http_parser)
     assert parse('http:one', 'etag', None) == (
         'httpp-http',
-        'http:one',
+        ['http:one'],
         'etag',
         None,
         'type/http',
@@ -974,7 +973,7 @@ def test_parser_selection():
     parse.mount_parser_by_mime_type(file_parser, 'type/file, text/plain;q=0.8')
     assert parse('file:one', None, 'last-modified') == (
         'filep-file',
-        'file:one',
+        ['file:one'],
         None,
         'last-modified',
         'type/file',
@@ -1007,23 +1006,23 @@ def test_parser_selection():
     parse.mount_parser_by_mime_type(make_dummy_parser('fallbackp-'), '*/*')
     assert parse('nomt:one') == (
         'fallbackp-nomt',
-        'nomt:one',
+        ['nomt:one'],
         None,
         None,
         'application/octet-stream',
     )
     assert parse('unkn:one') == (
         'fallbackp-unkn',
-        'unkn:one',
+        ['unkn:one'],
         None,
         None,
         'type/unknown',
     )
-    assert nomt_retriever.last_http_accept == 'type/http,type/file,*/*,text/plain;q=0.8'
+    assert nomt_retriever.last_http_accept == 'type/http,type/file,text/plain;q=0.8,*/*'
 
-    assert parse('file:o') == ('urlp-file', 'file:o', None, None, 'type/file')
+    assert parse('file:o') == ('urlp-file', ['file:o'], None, None, 'type/file')
     assert file_retriever.last_http_accept is None
-    assert parse('file:///o') == ('urlp-file', 'file:///o', None, None, 'type/file')
+    assert parse('file:///o') == ('urlp-file', ['file:///o'], None, None, 'type/file')
 
 
 def test_retriever_selection():
@@ -1035,14 +1034,14 @@ def test_retriever_selection():
 
     assert parse('http://generic.com/', 'etag', None) == (
         'generic',
-        'http://generic.com/',
+        ['http://generic.com/'],
         'etag',
         None,
         'type/subtype',
     )
     assert parse('http://specific.com/', None, 'last-modified') == (
         'specific',
-        'http://specific.com/',
+        ['http://specific.com/'],
         None,
         'last-modified',
         'type/subtype',
