@@ -1,9 +1,10 @@
+import os
 import threading
 from datetime import timezone
 
 import pytest
 from fakeparser import Parser
-from utils import naive_datetime
+from test_reader_counts import entries_per_day
 from utils import rename_argument
 from utils import utc_datetime
 from utils import utc_datetime as datetime
@@ -19,8 +20,11 @@ from reader import ReaderError
 from reader import SearchError
 from reader import SearchNotEnabledError
 from reader import StorageError
-from reader._search import Search
 from reader._storage import Storage
+from reader._storage._search import Search
+from reader._types import Action
+from reader._types import Change
+from reader.exceptions import ChangeTrackingNotEnabledError
 
 
 @pytest.fixture(params=[False, True], ids=['without_entries', 'with_entries'])
@@ -96,7 +100,7 @@ def test_update_search(reader):
 def test_update_search_fails_if_not_enabled(reader):
     with pytest.raises(SearchNotEnabledError) as excinfo:
         reader.update_search()
-    assert excinfo.value.__cause__ is None
+    assert isinstance(excinfo.value.__cause__, ChangeTrackingNotEnabledError)
     assert excinfo.value.message
 
 
@@ -384,7 +388,9 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
     if anything else except the indexed fields changes.
 
     """
-    from reader._search import Search
+    # TODO: maybe move this to a test_changes.py test
+
+    from reader._storage._search import Search
 
     strip_html_called = 0
 
@@ -396,7 +402,7 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
             return Search.strip_html(*args, **kwargs)
 
     # TODO: remove monkeypatching when make_reader() gets a search_cls argument
-    monkeypatch.setattr('reader.core.Search', MySearch)
+    monkeypatch.setattr('reader.core.Storage.make_search', lambda s: MySearch(s))
 
     reader = make_reader(db_path)
     reader._parser = parser = Parser()
@@ -428,52 +434,18 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
     feed = parser.feed(
         1, datetime(2010, 1, 2), title='feed', link='link', author='author'
     )
-
-    """
     entry = parser.entry(
-        1, 1, datetime(2010, 1, 2),
+        1,
+        1,
+        datetime(2010, 1, 2),
         title='entry',
         summary='summary',
         content=[Content('content')],
-        link='link', author='author',
+        link='link',
+        author='author',
         published=datetime(2010, 1, 2),
         enclosures=[Enclosure('enclosure')],
     )
-    """
-    # NOTE: As of 1.4, updating entries normally (above) uses INSERT OR REPLACE.
-    # REPLACE == DELETE + INSERT (https://www.sqlite.org/lang_conflict.html),
-    # so updating the entry normally *will not* fire the ON UPDATE trigger,
-    # but the ON DELETE and ON INSERT ones (basically, the ON UPDATE trigger
-    # never fires at the moment).
-    #
-    # Meanwhile, we do a (more intrusive/brittle) manual update:
-    with reader._search.get_db() as db:
-        db.execute(
-            """
-            UPDATE entries
-            SET (
-                title,
-                link,
-                updated,
-                author,
-                published,
-                summary,
-                content,
-                enclosures
-            ) = (
-                'entry',
-                'http://www.example.com/entries/1',
-                '2010-01-02 00:00:00',
-                'author',
-                '2010-01-02 00:00:00',
-                'summary',
-                '[{"value": "content", "type": null, "language": null}]',
-                '[{"href": "enclosure", "type": null, "length": null}]'
-            )
-            WHERE (id, feed) = ('1, 1', '1');
-            """
-        )
-    # TODO: Change this test when updating entries uses UPDATE instead of INSERT OR REPLACE
 
     reader.mark_entry_as_read(entry)
     reader.mark_entry_as_important(entry)
@@ -490,12 +462,111 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
     assert strip_html_called == 0
 
 
+@pytest.mark.parametrize('changes_limit', [None, 2])
+def test_update_unknown_changes_are_marked_as_done(reader, changes_limit):
+    reader._parser = parser = Parser()
+    reader.add_feed(parser.feed(1))
+    one = parser.entry(1, '1')
+    two = parser.entry(1, '2')
+    reader.update_feeds()
+
+    reader.enable_search()
+
+    changes = [
+        Change(Action.INSERT, b'wrong', two.resource_id),
+        Change(Action.INSERT, b'inexistent', one.resource_id),
+        Change(Action.DELETE, b'inexistent', one.resource_id),
+        Change(Action.INSERT, b'seq', ('1',)),
+        Change(Action.DELETE, b'seq', (), 'tag'),
+        Change(Action.INSERT, b'seq', ('1',), 'tag'),
+        Change(Action.DELETE, b'seq', ('1', '1'), 'tag'),
+    ]
+    changes += reader._storage.changes.get()
+    reader._storage.delete_entries([one.resource_id])
+    changes += reader._storage.changes.get()
+
+    seen_changes = set()
+
+    def get(action=None):
+        rv = list(changes)
+        if action is not None:
+            rv = [c for c in rv if c.action == action and c not in seen_changes]
+        if changes_limit:
+            rv = rv[:changes_limit]
+        seen_changes.update(rv)
+        return rv
+
+    done_changes = set()
+
+    def done(changes):
+        done_changes.update(changes)
+
+    reader._storage.changes.get = get
+    reader._storage.changes.done = done
+
+    reader.update_search()
+
+    assert set(changes) == done_changes
+
+
+def test_update_actually_deletes_from_database(reader):
+    # TODO: this is implementation-dependent, move to test_search.py?
+    reader._parser = parser = Parser()
+    reader.add_feed(parser.feed(1))
+    parser.entry(1, '1')
+    parser.entry(1, '2', summary='summary', content=[Content('content')])
+    reader.update_feeds()
+
+    db = reader._search.get_db()
+
+    def get_entries_search():
+        return sorted(
+            db.execute('select _feed, _id, _content_path from entries_search')
+        )
+
+    def get_entries_search_sync_state():
+        return sorted(
+            db.execute(
+                """
+            select feed, id, sequence, json_array_length(es_rowids)
+            from entries_search_sync_state
+        """
+            )
+        )
+
+    reader.update_search()
+    one, two = sorted(reader.get_entries(), key=lambda e: e.resource_id)
+    assert get_entries_search() == [
+        ('1', '1', None),
+        ('1', '2', '.content[0].value'),
+        ('1', '2', '.summary'),
+    ]
+    assert get_entries_search_sync_state() == [
+        ('1', '1', one._sequence, 1),
+        ('1', '2', two._sequence, 2),
+    ]
+
+    reader._storage.delete_entries([two.resource_id])
+    reader.update_search()
+    assert get_entries_search() == [
+        ('1', '1', None),
+    ]
+    assert get_entries_search_sync_state() == [
+        ('1', '1', one._sequence, 1),
+    ]
+
+    reader.delete_feed('1')
+    reader.update_search()
+    assert get_entries_search() == []
+    assert get_entries_search_sync_state() == []
+
+
 @with_sort
 def test_search_entries_basic(reader, sort):
     parser = Parser()
     reader._parser = parser
     # we're far intro the future, there are no recent entries
-    reader._now = lambda: naive_datetime(2020, 1, 1)
+    reader._now = lambda: datetime(2020, 1, 1)
 
     feed = parser.feed(1, datetime(2010, 1, 1))
     one = parser.entry(1, 1, datetime(2010, 1, 1), title='one')
@@ -619,9 +690,6 @@ def test_search_entries_basic(reader, sort):
 # search_entries() filtering is tested in test_reader.py::test_entries_filtering{,_error}
 
 
-from test_reader import entries_per_day
-
-
 @pytest.mark.parametrize(
     'query, expected',
     [
@@ -653,7 +721,7 @@ def test_search_entry_counts_basic(reader, query, expected):
     reader.enable_search()
     reader.update_search()
 
-    reader._now = lambda: naive_datetime(2010, 12, 31)
+    reader._now = lambda: datetime(2010, 12, 31)
 
     assert reader.search_entry_counts(query).averages == expected
 
@@ -824,233 +892,6 @@ def test_search_entries_sort_error(reader):
         set(reader.search_entries('one', sort='bad sort'))
 
 
-# BEGIN concurrency tests
-
-
-def test_update_search_entry_changed_during_strip_html(
-    db_path, make_reader, monkeypatch
-):
-    """Test the entry can't remain out of sync if it changes
-    during reader.update_search() in a strip_html() call.
-
-    https://github.com/lemon24/reader/issues/175#issuecomment-652489019
-
-    """
-    # This is a very intrusive test, maybe we should move it somewhere else.
-
-    reader = make_reader(db_path)
-    parser = reader._parser = Parser()
-
-    feed = parser.feed(1, datetime(2010, 1, 1), title='one')
-    parser.entry(1, 1, datetime(2010, 1, 1), title='one')
-    reader.add_feed(feed.url)
-    reader.update_feeds()
-
-    reader.enable_search()
-    reader.update_search()
-
-    feed = parser.feed(1, datetime(2010, 1, 2), title='two')
-    parser.entry(1, 1, datetime(2010, 1, 2), title='two')
-    reader.update_feed(feed.url)
-
-    in_strip_html = threading.Event()
-    can_return_from_strip_html = threading.Event()
-
-    def target():
-        # can't use fixture because it would run close() in a different thread
-        from reader import make_reader
-        from reader._search import Search
-
-        # strip_html() may or may not be used a SQLite user-defined function,
-        # hence the whole subclassing thing
-        class MySearch(Search):
-            @staticmethod
-            def strip_html(*args, **kwargs):
-                in_strip_html.set()
-                can_return_from_strip_html.wait()
-                return Search.strip_html(*args, **kwargs)
-
-        # TODO: remove monkeypatching when make_reader() gets a search_cls argument
-        monkeypatch.setattr('reader.core.Search', MySearch)
-
-        reader = make_reader(db_path)
-        try:
-            reader.update_search()
-        finally:
-            reader.close()
-
-    thread = threading.Thread(target=target)
-    thread.start()
-
-    in_strip_html.wait()
-
-    try:
-        feed = parser.feed(1, datetime(2010, 1, 3), title='three')
-        parser.entry(1, 1, datetime(2010, 1, 3), title='three')
-        reader._storage.get_db().execute("PRAGMA busy_timeout = 0;")
-        reader.update_feed(feed.url)
-        expected_title = 'three'
-    except StorageError:
-        expected_title = 'two'
-    finally:
-        can_return_from_strip_html.set()
-        thread.join()
-
-    reader.update_search()
-
-    (entry,) = reader.get_entries()
-    (result,) = reader.search_entries('one OR two OR three')
-    assert entry.title == result.metadata['.title'].value == expected_title
-
-
-def test_update_search_entry_changed_between_insert_loops(
-    db_path, make_reader, monkeypatch
-):
-    """Test the entry can't be added twice to the search index if it changes
-    during reader.update_search() between two insert loops.
-
-    The scenario is:
-
-    * entry has to_update set
-    * _delete_from_search removes it from search
-    * loop 1 of _insert_into_search finds entry and inserts it into search,
-      clears to_update
-    * entry has to_update set (if to_update is set because the feed changed,
-      last_updated does not change; even if it did, it doesn't matter,
-      since the transaction only spans a single loop)
-    * loop 2 of _insert_into_search finds entry and inserts it into search
-      again, clears to_update
-    * loop 3 of _insert_into_search doesn't find any entry, returns
-
-    https://github.com/lemon24/reader/issues/175#issuecomment-654213853
-
-    """
-    # This is a very intrusive test, maybe we should move it somewhere else.
-
-    reader = make_reader(db_path)
-    reader.enable_search()
-
-    parser = reader._parser = Parser()
-
-    feed = parser.feed(1, datetime(2010, 1, 1))
-    parser.entry(1, 1, datetime(2010, 1, 1), summary='one')
-    reader.add_feed(feed.url)
-    reader.update_feeds()
-
-    in_insert_chunk = threading.Event()
-    can_return_from_insert_chunk = threading.Event()
-
-    def target():
-        # can't use fixture because it would run close() in a different thread
-        from reader import make_reader
-
-        reader = make_reader(db_path)
-        original_insert_chunk = reader._search._insert_into_search_one_chunk
-
-        loop = 0
-
-        def insert_chunk(*args, **kwargs):
-            nonlocal loop
-            if loop == 1:
-                in_insert_chunk.set()
-                can_return_from_insert_chunk.wait()
-            loop += 1
-            return original_insert_chunk(*args, **kwargs)
-
-        reader._search._insert_into_search_one_chunk = insert_chunk
-
-        try:
-            reader.update_search()
-        finally:
-            reader.close()
-
-    thread = threading.Thread(target=target)
-    thread.start()
-
-    in_insert_chunk.wait()
-
-    try:
-        feed = parser.feed(1, datetime(2010, 1, 2))
-        parser.entry(1, 1, datetime(2010, 1, 2), summary='two')
-        reader.update_feed(feed.url)
-    finally:
-        can_return_from_insert_chunk.set()
-        thread.join()
-
-    (result,) = reader.search_entries('entry')
-    assert len(result.content) == 1
-
-    ((rowcount,),) = reader._search.get_db().execute(
-        "select count(*) from entries_search;"
-    )
-    assert rowcount == 1
-
-
-def test_update_search_concurrent_calls(db_path, make_reader, monkeypatch):
-    """Test concurrent calls to reader.update_search() don't interfere
-    with one another.
-
-    https://github.com/lemon24/reader/issues/175#issuecomment-652489019
-
-    """
-    # This is a very intrusive test, maybe we should move it somewhere else.
-
-    reader = make_reader(db_path)
-    parser = reader._parser = Parser()
-
-    feed = parser.feed(1, datetime(2010, 1, 1), title='feed')
-    parser.entry(
-        1,
-        1,
-        datetime(2010, 1, 1),
-        title='entry',
-        summary='summary',
-        content=[Content('content')],
-    )
-    reader.add_feed(feed.url)
-    reader.update_feeds()
-    reader.enable_search()
-
-    barrier = threading.Barrier(2)
-
-    def target():
-        # can't use fixture because it would run close() in a different thread
-        from reader import make_reader
-        from reader._search import Search
-
-        class MySearch(Search):
-            @staticmethod
-            def strip_html(*args, **kwargs):
-                barrier.wait()
-                return Search.strip_html(*args, **kwargs)
-
-        # TODO: remove monkeypatching when make_reader() gets a search_cls argument
-        monkeypatch.setattr('reader.core.Search', MySearch)
-
-        reader = make_reader(db_path)
-        try:
-            reader.update_search()
-        finally:
-            reader.close()
-
-    threads = [threading.Thread(target=target) for _ in range(2)]
-    for thread in threads:
-        thread.start()
-    for thread in threads:
-        thread.join()
-
-    (result,) = reader.search_entries('entry')
-    assert len(result.content) == 2
-
-    ((rowcount,),) = reader._search.get_db().execute(
-        "select count(*) from entries_search;"
-    )
-    assert rowcount == 2
-
-
-# END concurrency tests
-
-
 def test_add_entry_repeatedly(reader):
     # this could have been found with Hypothesis
 
