@@ -1,29 +1,36 @@
 import sqlite3
 import sys
+import threading
 from dataclasses import dataclass
 from functools import wraps
 
 import pytest
 from utils import rename_argument
 
-from reader._sqlite_utils import DBError
-from reader._sqlite_utils import ddl_transaction
-from reader._sqlite_utils import HeavyMigration
-from reader._sqlite_utils import IdError
-from reader._sqlite_utils import IntegrityError
-from reader._sqlite_utils import require_functions
-from reader._sqlite_utils import require_version
-from reader._sqlite_utils import RequirementError
-from reader._sqlite_utils import SchemaVersionError
-from reader._sqlite_utils import setup_db
-from reader._sqlite_utils import wrap_exceptions
+from reader._storage._sqlite_utils import DBError
+from reader._storage._sqlite_utils import ddl_transaction
+from reader._storage._sqlite_utils import ensure_application_id
+from reader._storage._sqlite_utils import HeavyMigration
+from reader._storage._sqlite_utils import IdError
+from reader._storage._sqlite_utils import IntegrityError
+from reader._storage._sqlite_utils import LocalConnectionFactory
+from reader._storage._sqlite_utils import require_functions
+from reader._storage._sqlite_utils import require_version
+from reader._storage._sqlite_utils import RequirementError
+from reader._storage._sqlite_utils import SchemaVersionError
+from reader._storage._sqlite_utils import setup_db
+from reader._storage._sqlite_utils import UsageError
+from reader._storage._sqlite_utils import wrap_exceptions
 
 
 original_sqlite3_connect = sqlite3.connect
 
 
 @pytest.fixture(autouse=True)
-def patch_sqlite3_connect(monkeypatch, request):
+def autoclose(monkeypatch, request):
+    if 'noautoclose' in request.keywords:
+        return
+
     @wraps(original_sqlite3_connect)
     def connect(*args, **kwargs):
         db = original_sqlite3_connect(*args, **kwargs)
@@ -150,28 +157,18 @@ def update_from_1_to_2_error(db):
     raise WeirdError('update')
 
 
-@dataclass
-class NewDefaultIdMigration(HeavyMigration):
-    id: int = 1234
-
-
-@pytest.fixture(params=[False, True])
-def migration_cls(request):
-    return HeavyMigration if not request.param else NewDefaultIdMigration
-
-
-def test_migration_create(migration_cls):
+def test_migration_create():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_2, 2, {})
+    migration = HeavyMigration(create_db_2, 2, {})
     # should call migration.create but not migration.migrations[1]
     migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one', 'two']
 
 
-def test_migration_create_error(migration_cls):
+def test_migration_create_error():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_2_error, 2, {})
+    migration = HeavyMigration(create_db_2_error, 2, {})
     # should call migration.create but not migration.migrations[1]
     with pytest.raises(WeirdError) as excinfo:
         migration.migrate(db)
@@ -180,33 +177,33 @@ def test_migration_create_error(migration_cls):
     assert columns == []
 
 
-def test_migration_update(migration_cls):
+def test_migration_update():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_1, 1, {})
+    migration = HeavyMigration(create_db_1, 1, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_2_error, 2, {1: update_from_1_to_2})
+    migration = HeavyMigration(create_db_2_error, 2, {1: update_from_1_to_2})
     # should call migration.migrations[1] but not migration.create
     migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one', 'two']
 
 
-def test_migration_no_update(migration_cls):
+def test_migration_no_update():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_2, 2, {})
+    migration = HeavyMigration(create_db_2, 2, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_2_error, 2, {})
+    migration = HeavyMigration(create_db_2_error, 2, {})
     # should call neither migration.create nor migration.migrations[1]
     migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one', 'two']
 
 
-def test_migration_update_error(migration_cls):
+def test_migration_update_error():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_1, 1, {})
+    migration = HeavyMigration(create_db_1, 1, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_2_error, 2, {1: update_from_1_to_2_error})
+    migration = HeavyMigration(create_db_2_error, 2, {1: update_from_1_to_2_error})
     # should call migration.migrations[1] but not migration.create
     with pytest.raises(WeirdError) as excinfo:
         migration.migrate(db)
@@ -215,40 +212,40 @@ def test_migration_update_error(migration_cls):
     assert columns == ['one']
 
 
-def test_migration_unsupported_old_version(migration_cls):
+def test_migration_unsupported_old_version():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_1, 1, {})
+    migration = HeavyMigration(create_db_1, 1, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_2, 2, {})
+    migration = HeavyMigration(create_db_2, 2, {})
     with pytest.raises(SchemaVersionError) as excinfo:
         migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one']
 
 
-def test_migration_unsupported_intermediary_version(migration_cls):
+def test_migration_unsupported_intermediary_version():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_1, 1, {})
+    migration = HeavyMigration(create_db_1, 1, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_2, 3, {1: update_from_1_to_2})
+    migration = HeavyMigration(create_db_2, 3, {1: update_from_1_to_2})
     with pytest.raises(SchemaVersionError) as excinfo:
         migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one']
 
 
-def test_migration_invalid_version(migration_cls):
+def test_migration_invalid_version():
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_2, 2, {})
+    migration = HeavyMigration(create_db_2, 2, {})
     migration.migrate(db)
-    migration = migration_cls(create_db_1, 1, {})
+    migration = HeavyMigration(create_db_1, 1, {})
     with pytest.raises(SchemaVersionError) as excinfo:
         migration.migrate(db)
     columns = [r[1] for r in db.execute("PRAGMA table_info(t);")]
     assert columns == ['one', 'two']
 
 
-def test_migration_integrity_error(migration_cls):
+def test_migration_integrity_error():
     def create_db(db):
         db.execute("CREATE TABLE t (one INTEGER PRIMARY KEY);")
         db.execute(
@@ -261,61 +258,60 @@ def test_migration_integrity_error(migration_cls):
     db = sqlite3.connect(':memory:')
     db.execute("PRAGMA foreign_keys = ON;")
 
-    migration = migration_cls(create_db, 1, {})
+    migration = HeavyMigration(create_db, 1, {})
     migration.migrate(db)
-    migration = migration_cls(create_db, 2, {1: update_from_1_to_2})
+    migration = HeavyMigration(create_db, 2, {1: update_from_1_to_2})
     with pytest.raises(IntegrityError):
         migration.migrate(db)
 
 
 @pytest.mark.parametrize('version', [-1, []])
-def test_migration_version_valuerror(migration_cls, version):
+def test_migration_version_valuerror(version):
     db = sqlite3.connect(':memory:')
-    migration = migration_cls(create_db_1, version, {})
+    migration = HeavyMigration(create_db_1, version, {})
     with pytest.raises(ValueError) as excinfo:
         migration.migrate(db)
 
 
-def test_nonempty(migration_cls):
+def test_migration_nonempty():
     db = sqlite3.connect(':memory:')
     db.execute("CREATE TABLE unexpected (one INTEGER);")
-    migration = migration_cls(create_db_2, 2, {})
+    migration = HeavyMigration(create_db_2, 2, {})
     with pytest.raises(DBError):
         migration.migrate(db)
 
 
-def test_invalid_id(migration_cls):
+def test_application_id_not_set():
+    db = sqlite3.connect(':memory:')
+    assert ensure_application_id(db, b'\x00\x00\x00\x02') is True
+    assert db.execute("PRAGMA application_id;").fetchone() == (2,)
+
+
+def test_application_id_already_set():
     db = sqlite3.connect(':memory:')
     db.execute("PRAGMA application_id = 2;")
-    migration = migration_cls(create_db_2, 2, {}, id=1)
-    with pytest.raises(IdError):
-        migration.migrate(db)
+    assert ensure_application_id(db, b'\x00\x00\x00\x02') is False
+    assert db.execute("PRAGMA application_id;").fetchone() == (2,)
 
 
-def test_missing_id_with_version(migration_cls):
+def test_application_id_invalid():
     db = sqlite3.connect(':memory:')
-    migration_cls.set_version(db, 2)
-    migration = migration_cls(create_db_2, 2, {}, id=1)
+    db.execute("PRAGMA application_id = 2;")
     with pytest.raises(IdError):
-        migration.migrate(db)
+        ensure_application_id(db, b'\x00\x00\x00\x01')
 
 
-def test_missing_id_after_migration(migration_cls):
+def test_application_id_nonempty():
     db = sqlite3.connect(':memory:')
+    db.execute("CREATE TABLE unexpected (one INTEGER);")
+    with pytest.raises(DBError):
+        ensure_application_id(db, b'\x00\x00\x00\x02')
 
-    migration = migration_cls(create_db_1, 1, {})
-    migration.migrate(db)
 
-    migration = migration_cls(create_db_2_error, 2, {1: update_from_1_to_2}, id=1)
-    with pytest.raises(IdError):
-        migration.migrate(db)
-
-    def bad_update_from_1_to_2(db):
-        migration_cls.set_id(db, 2)
-
-    migration = migration_cls(create_db_2_error, 2, {1: bad_update_from_1_to_2}, id=1)
-    with pytest.raises(IdError):
-        migration.migrate(db)
+def test_application_id_valueerror():
+    db = sqlite3.connect(':memory:')
+    with pytest.raises(ValueError):
+        ensure_application_id(db, b'\x00\x00')
 
 
 def test_require_version():
@@ -344,7 +340,7 @@ class MockConnection:
 
 
 def test_require_functions(monkeypatch):
-    from reader._sqlite_utils import FUNCTION_TESTS
+    from reader._storage._sqlite_utils import FUNCTION_TESTS
 
     db = sqlite3.connect(':memory:')
 
@@ -368,22 +364,61 @@ def test_require_functions(monkeypatch):
         require_functions(db, ['missing_function', 'bad_sql'])
 
 
-@pytest.mark.parametrize(
-    'wal_enabled, expected_mode', [(None, 'memory'), (True, 'wal'), (False, 'delete')]
-)
-def test_setup_db_wal_enabled(db_path, wal_enabled, expected_mode):
+def test_setup_db_wal_enabled(db_path):
     db = sqlite3.connect(db_path)
     db.execute("PRAGMA journal_mode = MEMORY;").close()
 
-    setup_db(
-        db,
-        create=lambda db: None,
-        version=1,
-        migrations={},
-        id=1234,
-        minimum_sqlite_version=(3, 15, 0),
-        wal_enabled=wal_enabled,
-    )
+    setup_db(db, id=b'1234')
 
-    ((mode,),) = db.execute("PRAGMA journal_mode;")
-    assert mode.lower() == expected_mode
+    cursor = db.execute("PRAGMA journal_mode;")
+    assert cursor.fetchone()[0].lower() == 'wal'
+    cursor.close()
+
+    db.execute("PRAGMA journal_mode = DELETE;").close()
+
+    setup_db(db, id=b'1234')
+
+    cursor = db.execute("PRAGMA journal_mode;")
+    assert cursor.fetchone()[0].lower() == 'delete'
+    cursor.close()
+
+
+@pytest.mark.slow
+@pytest.mark.noautoclose
+def test_factory_attach(db_path):
+    attached_path = db_path + '.attached'
+    factory = LocalConnectionFactory(db_path)
+
+    def get_databases():
+        nonlocal rv
+        rv = dict(factory().execute("select name, file from pragma_database_list()"))
+
+    factory.attach('attached', attached_path)
+
+    expected = {'main': db_path, 'attached': attached_path}
+
+    rv = None
+    get_databases()
+    assert rv == expected
+
+    rv = None
+    t = threading.Thread(target=get_databases)
+    t.start()
+    t.join()
+    assert rv == expected
+
+    with pytest.raises(UsageError):
+        factory.attach('another', attached_path)
+
+    def attach_second():
+        nonlocal rv
+        try:
+            factory.attach('second', attached_path)
+        except Exception as e:
+            rv = e
+
+    rv = None
+    t = threading.Thread(target=attach_second)
+    t.start()
+    t.join()
+    assert isinstance(rv, UsageError), rv
