@@ -2,24 +2,24 @@ import os
 import sqlite3
 import sys
 import threading
-from datetime import datetime
 from unittest.mock import ANY
 from unittest.mock import MagicMock
 
 import pytest
 from utils import rename_argument
+from utils import utc_datetime as datetime
 
-import reader._sqlite_utils
+import reader._storage._sqlite_utils
 from reader import EntryNotFoundError
 from reader import FeedNotFoundError
 from reader import InvalidSearchQueryError
 from reader import StorageError
-from reader._sqlite_utils import DBError
-from reader._sqlite_utils import HeavyMigration
-from reader._sqlite_utils import require_version
 from reader._storage import Storage
+from reader._storage._sqlite_utils import DBError
+from reader._storage._sqlite_utils import HeavyMigration
+from reader._storage._sqlite_utils import require_version
 from reader._types import EntryData
-from reader._types import EntryFilterOptions
+from reader._types import EntryFilter
 from reader._types import EntryForUpdate
 from reader._types import EntryUpdateIntent
 from reader._types import FeedData
@@ -34,7 +34,7 @@ def test_storage_errors_connect(tmp_path):
     assert 'while opening' in excinfo.value.message
 
 
-@pytest.mark.parametrize('db_error_cls', reader._sqlite_utils.db_errors)
+@pytest.mark.parametrize('db_error_cls', reader._storage._sqlite_utils.db_errors)
 def test_db_errors(db_path, db_error_cls):
     """..._sqlite_utils.DBError subclasses should be wrapped in StorageError."""
 
@@ -130,7 +130,7 @@ def test_timeout(db_path, monkeypatch):
     assert connect.expected_timeout == 19
 
 
-def test_close():
+def test_close(monkeypatch):
     close_called = False
 
     class Connection(sqlite3.Connection):
@@ -139,7 +139,9 @@ def test_close():
             nonlocal close_called
             close_called = True
 
-    storage = Storage(':memory:', factory=Connection)
+    monkeypatch.setattr('reader._storage._base.CONNECTION_CLS', Connection)
+
+    storage = Storage(':memory:')
     storage.get_db().execute('values (1)')
 
     storage.close()
@@ -149,14 +151,16 @@ def test_close():
     storage.close()
 
 
-def test_close_error():
+def test_close_error(monkeypatch):
     class Connection(sqlite3.Connection):
         pass
 
     def execute(*args):
         raise sqlite3.ProgrammingError('unexpected error')
 
-    storage = Storage(':memory:', factory=Connection)
+    monkeypatch.setattr('reader._storage._base.CONNECTION_CLS', Connection)
+
+    storage = Storage(':memory:')
     storage.get_db().execute = execute
 
     # should not be StorageError, because it's likely a bug
@@ -196,16 +200,16 @@ def set_feed_updates_enabled(storage, feed, __):
     storage.set_feed_updates_enabled(feed.url, 1)
 
 
-def mark_as_stale(storage, feed, __):
-    storage.mark_as_stale(feed.url)
+def set_feed_stale(storage, feed, __):
+    storage.set_feed_stale(feed.url, True)
 
 
-def mark_as_read(storage, feed, entry):
-    storage.mark_as_read(feed.url, entry.id, 1, None)
+def set_entry_read(storage, feed, entry):
+    storage.set_entry_read(entry.resource_id, 1, None)
 
 
-def mark_as_important(storage, feed, entry):
-    storage.mark_as_important(feed.url, entry.id, 1, None)
+def set_entry_important(storage, feed, entry):
+    storage.set_entry_important(entry.resource_id, 1, None)
 
 
 def get_entry_recent_sort(storage, feed, entry):
@@ -266,12 +270,8 @@ def delete_entries(storage, feed, entry):
     storage.delete_entries([entry.resource_id])
 
 
-def get_entries_chunk_size_0(storage, _, __):
-    list(storage.get_entries_page(chunk_size=0, now=datetime(2010, 1, 1)))
-
-
-def get_entries_chunk_size_1(storage, _, __):
-    list(storage.get_entries_page(chunk_size=1, now=datetime(2010, 1, 1)))
+def get_entries(storage, _, __):
+    list(storage.get_entries())
 
 
 def get_tags(storage, feed, __):
@@ -294,14 +294,6 @@ def get_entry_counts(storage, _, __):
     storage.get_entry_counts(now=datetime(2010, 1, 1)),
 
 
-def get_feed_last(storage, feed, __):
-    storage.get_feed_last('title', feed.url)
-
-
-def get_entry_last(storage, feed, entry):
-    storage.get_entry_last(datetime(2010, 1, 1), 'recent', (feed.url, entry.id))
-
-
 @pytest.mark.slow
 @pytest.mark.parametrize(
     'do_stuff',
@@ -314,9 +306,9 @@ def get_entry_last(storage, feed, entry):
         get_entries_for_update,
         set_feed_user_title,
         set_feed_updates_enabled,
-        mark_as_stale,
-        mark_as_read,
-        mark_as_important,
+        set_feed_stale,
+        set_entry_read,
+        set_entry_important,
         get_entry_recent_sort,
         set_entry_recent_sort,
         update_feed,
@@ -325,15 +317,12 @@ def get_entry_last(storage, feed, entry):
         add_or_update_entries,
         add_entry,
         delete_entries,
-        get_entries_chunk_size_0,
-        get_entries_chunk_size_1,
+        get_entries,
         get_tags,
         set_tag,
         delete_tag,
         get_feed_counts,
         get_entry_counts,
-        get_feed_last,
-        get_entry_last,
     ],
 )
 def test_errors_locked(db_path, do_stuff):
@@ -345,8 +334,11 @@ def test_errors_locked(db_path, do_stuff):
 def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
     """Actual implementation of test_errors_locked, so it can be reused."""
 
+    storage = Storage(db_path)
     # WAL provides more concurrency; some things won't to block with it enabled.
-    storage = Storage(db_path, wal_enabled=False)
+    storage.close()
+    storage.get_db().execute("PRAGMA journal_mode = DELETE;").close()
+
     storage.get_db().execute("PRAGMA busy_timeout = 0;")
 
     feed = FeedData('one')
@@ -368,7 +360,7 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
     can_return_from_transaction = threading.Event()
 
     def target():
-        storage = Storage(db_path, wal_enabled=False)
+        storage = Storage(db_path)
         db = storage.get_db()
         db.isolation_level = None
         db.execute("BEGIN EXCLUSIVE;")
@@ -377,7 +369,7 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
         db.execute("ROLLBACK;")
 
     if pre_stuff:
-        pre_stuff(storage, feed, entry)
+        pre_stuff(storage)
 
     thread = threading.Thread(target=target)
     thread.start()
@@ -394,31 +386,19 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
 
 
 def iter_get_feeds(storage):
-    return storage.get_feeds_page(chunk_size=1)
+    return storage.get_feeds()
 
 
 def iter_get_feeds_for_update(storage):
     return storage.get_feeds_for_update()
 
 
-def iter_pagination_chunk_size_0(storage):
-    return storage.get_entries_page(chunk_size=0, now=datetime(2010, 1, 1))
-
-
-def iter_pagination_chunk_size_1(storage):
-    return storage.get_entries_page(chunk_size=1, now=datetime(2010, 1, 1))
-
-
-def iter_pagination_chunk_size_2(storage):
-    return storage.get_entries_page(chunk_size=2, now=datetime(2010, 1, 1))
-
-
-def iter_pagination_chunk_size_3(storage):
-    return storage.get_entries_page(chunk_size=3, now=datetime(2010, 1, 1))
+def iter_get_entries(storage):
+    return storage.get_entries()
 
 
 def iter_get_tags(storage):
-    return storage.get_tags_page(('two',), chunk_size=1)
+    return storage.get_tags(('two',))
 
 
 @pytest.mark.slow
@@ -427,27 +407,23 @@ def iter_get_tags(storage):
     [
         iter_get_feeds,
         iter_get_feeds_for_update,
-        pytest.param(
-            iter_pagination_chunk_size_0,
-            marks=pytest.mark.xfail(raises=StorageError, strict=True),
-        ),
-        iter_pagination_chunk_size_1,
-        iter_pagination_chunk_size_2,
-        iter_pagination_chunk_size_3,
+        iter_get_entries,
         iter_get_tags,
     ],
 )
-def test_iter_locked(db_path, iter_stuff):
+def test_iter_locked(db_path, iter_stuff, chunk_size):
     """Methods that return an iterable shouldn't block the underlying storage
     if the iterable is not consumed."""
-    check_iter_locked(db_path, None, iter_stuff)
+    check_iter_locked(db_path, None, iter_stuff, chunk_size)
 
 
-def check_iter_locked(db_path, pre_stuff, iter_stuff):
+def check_iter_locked(db_path, pre_stuff, iter_stuff, chunk_size):
     """Actual implementation of test_errors_locked, so it can be reused."""
 
+    storage = Storage(db_path)
     # WAL provides more concurrency; some things won't to block with it enabled.
-    storage = Storage(db_path, wal_enabled=False)
+    storage.close()
+    storage.get_db().execute("PRAGMA journal_mode = DELETE;").close()
 
     feed = FeedData('one')
     entry = EntryData('one', 'entry', datetime(2010, 1, 1), title='entry')
@@ -485,10 +461,10 @@ def check_iter_locked(db_path, pre_stuff, iter_stuff):
     next(rv)
 
     # shouldn't raise an exception
-    storage = Storage(db_path, timeout=0, wal_enabled=False)
-    storage.mark_as_read(feed.url, entry.id, 1, None)
     storage = Storage(db_path, timeout=0)
-    storage.mark_as_read(feed.url, entry.id, 0, None)
+    storage.set_entry_read((feed.url, entry.id), 1, None)
+    storage = Storage(db_path, timeout=0)
+    storage.set_entry_read((feed.url, entry.id), 0, None)
 
 
 def test_update_feed_last_updated_not_found(db_path):
@@ -532,47 +508,37 @@ def storage_with_two_entries(storage):
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_unimportant_by_default(storage):
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important='nottrue')
-        )
-    } == {'one', 'two'}
+    assert {e.id for e in storage.get_entries(EntryFilter(important='nottrue'))} == {
+        'one',
+        'two',
+    }
 
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_get_entries(storage):
-    storage.mark_as_important('feed', 'one', True, datetime(2010, 1, 2))
+    storage.set_entry_important(('feed', 'one'), True, datetime(2010, 1, 2))
 
-    assert {e.id for e in storage.get_entries(now=datetime(2010, 1, 1))} == {
+    assert {e.id for e in storage.get_entries()} == {
         'one',
         'two',
     }
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important='any')
-        )
-    } == {'one', 'two'}
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important='istrue')
-        )
-    } == {'one'}
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important='nottrue')
-        )
-    } == {'two'}
+    assert {e.id for e in storage.get_entries(EntryFilter(important='any'))} == {
+        'one',
+        'two',
+    }
+    assert {e.id for e in storage.get_entries(EntryFilter(important='istrue'))} == {
+        'one'
+    }
+    assert {e.id for e in storage.get_entries(EntryFilter(important='nottrue'))} == {
+        'two'
+    }
 
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_entry_important(storage):
-    storage.mark_as_important('feed', 'one', True, None)
+    storage.set_entry_important(('feed', 'one'), True, None)
 
-    assert {e.id: e.important for e in storage.get_entries(datetime(2010, 1, 1))} == {
+    assert {e.id: e.important for e in storage.get_entries()} == {
         'one': True,
         'two': None,
     }
@@ -580,30 +546,25 @@ def test_important_entry_important(storage):
 
 @rename_argument('storage', 'storage_with_two_entries')
 def test_important_mark_as_unimportant(storage):
-    storage.mark_as_important('feed', 'one', True, None)
-    storage.mark_as_important('feed', 'one', False, None)
+    storage.set_entry_important(('feed', 'one'), True, None)
+    storage.set_entry_important(('feed', 'one'), False, None)
 
-    assert {
-        e.id
-        for e in storage.get_entries(
-            datetime(2010, 1, 1), EntryFilterOptions(important='istrue')
-        )
-    } == set()
+    assert {e.id for e in storage.get_entries(EntryFilter(important='istrue'))} == set()
 
 
 def test_important_mark_entry_not_found(storage):
     with pytest.raises(EntryNotFoundError):
-        storage.mark_as_important('feed', 'one', True, None)
+        storage.set_entry_important(('feed', 'one'), True, None)
 
 
 def test_minimum_sqlite_version(db_path, monkeypatch):
     mock = MagicMock(wraps=require_version, side_effect=DBError)
-    monkeypatch.setattr('reader._sqlite_utils.require_version', mock)
+    monkeypatch.setattr('reader._storage._sqlite_utils.require_version', mock)
 
     with pytest.raises(StorageError):
         Storage(db_path)
 
-    mock.assert_called_with(ANY, (3, 15))
+    mock.assert_called_with(ANY, (3, 18))
 
 
 MIGRATION_MINIMUM_VERSION = 29
@@ -640,3 +601,8 @@ def test_get_set_recent_sort(storage):
     with pytest.raises(EntryNotFoundError) as excinfo:
         storage.set_entry_recent_sort(('feed', 'xxx'), datetime(2010, 1, 1))
     assert excinfo.value.resource_id == ('feed', 'xxx')
+
+
+def test_application_id(storage):
+    id = storage.factory().execute('pragma application_id').fetchone()[0]
+    assert id == int.from_bytes(b'read', 'big')
