@@ -30,14 +30,7 @@ def test_namedtuple_compat():
         one: int
         two: int = None
 
-    assert Object._make((1, 2)) == Object(1, 2)
-    with pytest.raises(TypeError):
-        Object._make((1,))
-    with pytest.raises(TypeError):
-        Object._make((1, 2, 3))
-
     assert Object(1, 1)._replace(two=2) == Object(1, 2)
-
     assert Object(1, 2)._asdict() == {'one': 1, 'two': 2}
 
 
